from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.zscaler import zia_alert_context, zia_success


@panther_managed
class ZIAPasswordExpiration(Rule):
    id = "ZIA.Password.Expiration-prototype"
    default_description = "This rule detects when password expiration eas set/removed."
    display_name = "ZIA Password Expiration"
    default_runbook = (
        "Verify that this change was planned. If not, revert the change and ensure this doesn't happen again."
    )
    default_reference = "https://help.zscaler.com/zia/configuring-password-expiration"
    default_severity = Severity.MEDIUM
    reports = {"MITRE ATT&CK": ["TA0007:T1201"]}
    log_types = [LogType.ZSCALER_ZIA_ADMIN_AUDIT_LOG]

    def rule(self, event):
        if not zia_success(event):
            return False
        action = event.deep_get("event", "action", default="ACTION_NOT_FOUND")
        category = event.deep_get("event", "category", default="CATEGORY_NOT_FOUND")
        password_exp_pre = event.deep_get(
            "event",
            "preaction",
            "passwordExpirationEnabled",
            default="<PRE_PASSWORD_EXPIRATION_NOT_FOUND>",
        )
        password_exp_post = event.deep_get(
            "event",
            "postaction",
            "passwordExpirationEnabled",
            default="<POST_PASSWORD_EXPIRATION_NOT_FOUND>",
        )
        if action == "UPDATE" and category == "LOGIN" and (password_exp_pre != password_exp_post):
            return True
        return False

    def title(self, event):
        return f"[Zscaler.ZIA]: SAML configuration was changed by admin with id [{event.deep_get('event', 'adminid', default='<ADMIN_ID_NOT_FOUND>')}]"

    def alert_context(self, event):
        return zia_alert_context(event)

    tests = [
        RuleTest(
            name="Administration Management > Administrator Management > Set Password Expriration 180 days",
            expected_result=True,
            log={
                "event": {
                    "action": "UPDATE",
                    "adminid": "admin@16991311.zscalerbeta.net",
                    "auditlogtype": "ZIA",
                    "category": "LOGIN",
                    "clientip": "123.123.123.123",
                    "errorcode": "None",
                    "interface": "UI",
                    "postaction": {"passwordExpirationEnabled": True, "passwordExpiryDays": 180},
                    "preaction": {"passwordExpirationEnabled": False, "passwordExpiryDays": 180},
                    "recordid": "331",
                    "resource": "None",
                    "result": "SUCCESS",
                    "subcategory": "PASSWORD_EXPIRY",
                    "time": "2024-10-22 22:12:25.000000000",
                },
                "sourcetype": "zscalernss-audit",
            },
        ),
        RuleTest(
            name="Administration Management > Administrator Management > Remove Password Expriration",
            expected_result=True,
            log={
                "event": {
                    "action": "UPDATE",
                    "adminid": "admin@16991311.zscalerbeta.net",
                    "auditlogtype": "ZIA",
                    "category": "LOGIN",
                    "clientip": "123.123.123.123",
                    "errorcode": "None",
                    "interface": "UI",
                    "postaction": {"passwordExpirationEnabled": False, "passwordExpiryDays": 180},
                    "preaction": {"passwordExpirationEnabled": True, "passwordExpiryDays": 180},
                    "recordid": "331",
                    "resource": "None",
                    "result": "SUCCESS",
                    "subcategory": "PASSWORD_EXPIRY",
                    "time": "2024-10-22 22:12:25.000000000",
                },
                "sourcetype": "zscalernss-audit",
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed


@panther_managed
class AsanaWorkspaceFormLinkAuthRequirementDisabled(Rule):
    default_description = "An Asana Workspace Form Link is a unique URL that allows you to create a task directly within a specific Workspace or Project in Asana, using a web form. Disabling authentication requirements may allow unauthorized users to create tasks. "
    display_name = "Asana Workspace Form Link Auth Requirement Disabled"
    default_reference = "https://help.asana.com/hc/en-us/articles/14111697664923-Forms-access-permissions#:~:text=SSO%2C%20SAML%2C%20or-,no%20authentication%20method,-).%20If%20no%20authentication"
    default_severity = Severity.LOW
    log_types = [LogType.ASANA_AUDIT]
    id = "Asana.Workspace.Form.Link.Auth.Requirement.Disabled-prototype"

    def rule(self, event):
        return event.get("event_type") == "workspace_form_link_authentication_required_disabled"

    def title(self, event):
        workspace = event.deep_get("resource", "name", default="<WORKSPACE_NOT_FOUND>")
        actor = event.deep_get("actor", "email", default="<ACTOR_NOT_FOUND>")
        return f"Asana Workspace [{workspace}] Form Link Auth Requirement  was disabled by [{actor}]."

    tests = [
        RuleTest(
            name="form auth requirement disabled",
            expected_result=True,
            log={
                "actor": {
                    "actor_type": "user",
                    "email": "homer.simpson@simpsons.com",
                    "gid": "1234567890",
                    "name": "Homer Simpson",
                },
                "context": {
                    "client_ip_address": "1.2.3.4",
                    "context_type": "web",
                    "user_agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/108.0.0.0 Safari/537.36",
                },
                "created_at": "2022-12-16 19:32:00.922",
                "details": {},
                "event_category": "admin_settings",
                "event_type": "workspace_form_link_authentication_required_disabled",
                "gid": "1234567890",
                "resource": {"gid": "111234", "name": "Simpsons Lab", "resource_type": "workspace"},
            },
        ),
        RuleTest(
            name="other",
            expected_result=False,
            log={
                "actor": {
                    "actor_type": "user",
                    "email": "homer.simpson@panther.io",
                    "gid": "12345",
                    "name": "Homer Simpson",
                },
                "context": {
                    "client_ip_address": "12.12.12.12",
                    "context_type": "web",
                    "user_agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/108.0.0.0 Safari/537.36",
                },
                "created_at": "2022-12-16 19:35:21.026",
                "details": {"new_value": "public"},
                "event_category": "access_control",
                "event_type": "team_privacy_settings_changed",
                "gid": "12345",
                "resource": {"gid": "12345", "name": "Example Team Name", "resource_type": "team"},
            },
        ),
    ]



import re

from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.snowflake import query_history_alert_context


@panther_managed
class SnowflakeStreamUserCreated(Rule):
    id = "Snowflake.Stream.UserCreated-prototype"
    display_name = "Snowflake User Created"
    enabled = False
    log_types = [LogType.SNOWFLAKE_QUERY_HISTORY]
    default_severity = Severity.INFO
    reports = {"MITRE ATT&CK": ["TA0003:T1136"]}
    default_description = "Detect new users created in Snowflake."
    tags = ["Snowflake", "[MITRE] Persistence", "[MITRE] Create Account"]
    CREATE_USER_EXPR = re.compile("create user (\\w+).*", flags=re.I)
    CREATE_USER = ""

    def rule(self, event):
        self.CREATE_USER = self.CREATE_USER_EXPR.match(event.get("QUERY_TEXT", ""))
        return all(
            (
                event.get("EXECUTION_STATUS") == "SUCCESS",
                event.get("QUERY_TYPE") == "CREATE_USER",
                self.CREATE_USER is not None,
            ),
        )

    def title(self, event):
        new_user = self.CREATE_USER.group(1)
        actor = event.get("user_name", "<UNKNOWN ACTOR>")
        source = event.get("p_source_label", "<UNKNOWN SOURCE>")
        return f"{source}: Snowflake user {new_user} created by {actor}"

    def alert_context(self, event):
        return query_history_alert_context(event)

    tests = [
        RuleTest(
            name="User Created",
            expected_result=True,
            log={
                "p_event_time": "2024-10-09 19:43:05.007000000",
                "p_log_type": "Snowflake.QueryHistory",
                "BYTES_DELETED": 0,
                "EXECUTION_STATUS": "SUCCESS",
                "QUERY_TEXT": "CREATE USER MERCY\nPASSWORD = '☺☺☺☺☺'\nDEFAULT_ROLE = PUBLIC;",
                "QUERY_TYPE": "CREATE_USER",
                "ROLE_NAME": "ACCOUNTADMIN",
                "USER_NAME": "LEX_LUTHOR",
                "WAREHOUSE_NAME": "ADMIN_WH",
            },
        ),
    ]



from pypanther.rules.osquery.osquery_linux_aws_commands import (
    OsqueryLinuxAWSCommandExecuted as OsqueryLinuxAWSCommandExecuted,
)
from pypanther.rules.osquery.osquery_linux_logins_non_office import (
    OsqueryLinuxLoginFromNonOffice as OsqueryLinuxLoginFromNonOffice,
)
from pypanther.rules.osquery.osquery_linux_mac_vulnerable_xz_liblzma import (
    OsqueryLinuxMacVulnerableXZliblzma as OsqueryLinuxMacVulnerableXZliblzma,
)
from pypanther.rules.osquery.osquery_mac_application_firewall import (
    OsqueryMacApplicationFirewallSettings as OsqueryMacApplicationFirewallSettings,
)
from pypanther.rules.osquery.osquery_mac_enable_auto_update import (
    OsqueryMacAutoUpdateEnabled as OsqueryMacAutoUpdateEnabled,
)
from pypanther.rules.osquery.osquery_mac_osx_attacks import OsqueryMacOSXAttacks as OsqueryMacOSXAttacks
from pypanther.rules.osquery.osquery_mac_osx_attacks_keyboard_events import (
    OsqueryMacOSXAttacksKeyboardEvents as OsqueryMacOSXAttacksKeyboardEvents,
)
from pypanther.rules.osquery.osquery_mac_unwanted_chrome_extensions import (
    OsqueryMacUnwantedChromeExtensions as OsqueryMacUnwantedChromeExtensions,
)
from pypanther.rules.osquery.osquery_ossec import OsqueryOSSECRootkitDetected as OsqueryOSSECRootkitDetected
from pypanther.rules.osquery.osquery_outdated import OsqueryOutdatedAgent as OsqueryOutdatedAgent
from pypanther.rules.osquery.osquery_outdated_macos import OsqueryUnsupportedMacOS as OsqueryUnsupportedMacOS
from pypanther.rules.osquery.osquery_ssh_listener import OsquerySSHListener as OsquerySSHListener
from pypanther.rules.osquery.osquery_suspicious_cron import OsquerySuspiciousCron as OsquerySuspiciousCron



from pypanther.rules.sentinelone.sentinelone_alert_passthrough import (
    SentinelOneAlertPassthrough as SentinelOneAlertPassthrough,
)
from pypanther.rules.sentinelone.sentinelone_threats import SentinelOneThreats as SentinelOneThreats



from fnmatch import fnmatch

from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.okta import okta_alert_context


@panther_managed
class OktaRateLimits(Rule):
    default_description = "Potential DoS/Bruteforce attack or hitting limits (system degradation)"
    display_name = "Okta Rate Limits"
    default_severity = Severity.LOW
    tags = ["Credential Access", "Brute Force", "Impact", "Network Denial of Service"]
    reports = {"MITRE ATT&CK": ["TA0006:T1110", "TA0040:T1498"]}
    default_reference = "https://developer.okta.com/docs/reference/rl-system-log-events/"
    dedup_period_minutes = 1440
    log_types = [LogType.OKTA_SYSTEM_LOG]
    id = "Okta.Rate.Limits-prototype"
    DETECTION_EVENTS = [
        "app.oauth2.client_id_rate_limit_warning",
        "application.integration.rate_limit_exceeded",
        "system.client.rate_limit.*",
        "system.client.concurrency_rate_limit.*",
        "system.operation.rate_limit.*",
        "system.org.rate_limit.*",
        "core.concurrency.org.limit.violation",
    ]

    def rule(self, event):
        eventtype = event.get("eventtype", "")
        for detection_event in self.DETECTION_EVENTS:
            if fnmatch(eventtype, detection_event) and "violation" in eventtype:
                return True
        return False

    def title(self, event):
        actor = event.deep_get("actor", "alternateId")
        if actor == "unknown":
            actor = event.deep_get("actor", "displayName", default="<id-not-found>")
        return f"Okta Rate Limit Event: [{event.get('eventtype', '')}] by [{actor}/{event.deep_get('actor', 'type', default='<type-not-found>')}] "

    def dedup(self, event):
        return event.deep_get("actor", "id")

    def alert_context(self, event):
        return okta_alert_context(event)

    tests = [
        RuleTest(
            name="system.org.ratelimit.warning",
            expected_result=False,
            log={
                "actor": {
                    "alternateId": "homer.simpson@duff.com",
                    "displayName": "Homer Simpson",
                    "id": "00abc456",
                    "type": "User",
                },
                "authenticationcontext": {"authenticationStep": 0, "externalSessionId": "abc12345"},
                "client": {
                    "device": "Unknown",
                    "ipAddress": "1.2.3.4",
                    "userAgent": {"browser": "UNKNOWN", "os": "Unknown", "rawUserAgent": "Chrome"},
                    "zone": "null",
                },
                "debugcontext": {
                    "debugData": {
                        "rateLimitScopeType": "ORG",
                        "rateLimitSecondsToReset": "11",
                        "requestId": "abc1234",
                        "requestUri": "/api/v1/users",
                        "threshold": "500",
                        "timeSpan": "1",
                        "timeUnit": "MINUTES",
                        "url": "/api/v1/users?limit=100",
                        "userId": "abc1234",
                        "warningPercent": "80",
                    },
                },
                "displaymessage": "Rate limit warning",
                "eventtype": "system.org.rate_limit.warning",
                "legacyeventtype": "core.framework.ratelimit.warning",
                "outcome": {"result": "SUCCESS"},
                "published": "2022-08-26 07:51:24.601",
                "request": {"ipChain": [{"ip": "1.2.3.4", "version": "V4"}]},
                "securitycontext": {},
                "severity": "INFO",
                "target": [{"id": "/api/v1/users", "type": "URL Pattern"}, {"id": "abc12345", "type": "Bucket Uuid"}],
                "transaction": {"detail": {"requestApiTokenId": "A.12345.DEFGH"}, "id": "ABC1235", "type": "WEB"},
                "uuid": "abc-1234-abcd",
                "version": "0",
            },
        ),
        RuleTest(
            name="system.operation.ratelimit.violation",
            expected_result=True,
            log={
                "actor": {
                    "alternateId": "homer.simpson@duff.com",
                    "displayName": "Homer Simpson",
                    "id": "00abc456",
                    "type": "User",
                },
                "authenticationcontext": {"authenticationStep": 0, "externalSessionId": "abc12345"},
                "client": {
                    "device": "Unknown",
                    "ipAddress": "1.2.3.4",
                    "userAgent": {"browser": "UNKNOWN", "os": "Unknown", "rawUserAgent": "Chrome"},
                    "zone": "null",
                },
                "debugcontext": {
                    "debugData": {
                        "authnRequestId": "ABCDFDE",
                        "dtHash": "adfalsjflasfjsdfd",
                        "operationRateLimitScopeType": "User",
                        "operationRateLimitSecondsToReset": "10",
                        "operationRateLimitSubtype": "Authenticated user",
                        "operationRateLimitThreshold": "40",
                        "operationRateLimitTimeSpan": "10",
                        "operationRateLimitTimeUnit": "SECONDS",
                        "operationRateLimitType": "Web request",
                        "requestId": "asfsagadffdaf",
                        "requestUri": "/app/google/",
                        "url": "/app/google/",
                    },
                },
                "displaymessage": "Operation rate limit violation",
                "eventtype": "system.operation.rate_limit.violation",
                "outcome": {"reason": "Too many requests attempted by an individual user", "result": "DENY"},
                "published": "2022-08-29 16:07:26.592",
                "request": {"ipChain": [{"ip": "1.2.3.4", "version": "V4"}]},
                "securitycontext": {},
                "severity": "WARN",
                "target": [{"id": "/app/{app}/{key}/", "type": "URL Pattern"}],
                "transaction": {"detail": {}, "id": "YABCDE", "type": "WEB"},
                "uuid": "asdfdashh",
                "version": "0",
            },
        ),
        RuleTest(
            name="application.integration.rate_limit_exceeded",
            expected_result=False,
            log={
                "actor": {
                    "alternateId": "homer.simpson@duff.com",
                    "displayName": "Homer Simpson",
                    "id": "00abc456",
                    "type": "User",
                },
                "authenticationcontext": {"authenticationStep": 0, "externalSessionId": "abc12345"},
                "client": {
                    "device": "Unknown",
                    "ipAddress": "1.2.3.4",
                    "userAgent": {"browser": "UNKNOWN", "os": "Unknown", "rawUserAgent": "Chrome"},
                    "zone": "null",
                },
                "debugcontext": {"debugData": {}},
                "eventtype": "application.integration.rate_limit_exceeded",
                "legacyeventtype": "app.api.error.rate.limit.exceeded",
                "outcome": {"result": "SUCCESS"},
                "published": "2022-06-10 17:19:58.423",
                "request": {},
                "securitycontext": {},
                "severity": "INFO",
                "target": [{"alternateId": "App ", "displayName": "App", "id": "12345", "type": "AppInstance"}],
                "transaction": {"detail": {}, "id": "sdfg", "type": "JOB"},
                "uuid": "aaa-bb-ccc",
                "version": "0",
            },
        ),
        RuleTest(
            name="Non event",
            expected_result=False,
            log={
                "actor": {
                    "alternateId": "homer.simpson@duff.com",
                    "displayName": "Homer Simpsons",
                    "id": "00ABC123",
                    "type": "User",
                },
                "authenticationcontext": {"authenticationStep": 0, "externalSessionId": "xyz1234"},
                "client": {
                    "device": "Computer",
                    "geographicalContext": {
                        "city": "Springfield",
                        "country": "United States",
                        "geolocation": {"lat": 11.111, "lon": -70},
                        "postalCode": "1234",
                        "state": "California",
                    },
                    "ipAddress": "1.2.3.4",
                    "userAgent": {
                        "browser": "CHROME",
                        "os": "Mac OS X",
                        "rawUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/108.0.0.0 Safari/537.36",
                    },
                    "zone": "null",
                },
                "debugcontext": {
                    "debugData": {
                        "authnRequestId": "ABC123",
                        "deviceFingerprint": "009988771ABC",
                        "dtHash": "123abc1234",
                        "requestId": "abc-111-adf",
                        "requestUri": "/idp/idx/identify",
                        "threatSuspected": "false",
                        "url": "/idp/idx/identify?",
                    },
                },
                "displaymessage": "Group Privilege granted",
                "eventtype": "group.privilege.grant",
                "legacyeventtype": "group.privilege.grant",
                "outcome": {"result": "FAILURE"},
                "published": "2022-12-13 00:58:19.811",
                "request": {
                    "ipChain": [
                        {
                            "geographicalContext": {
                                "city": "Springfield",
                                "country": "United States",
                                "geolocation": {"lat": 11.111, "lon": -70},
                                "postalCode": "1234",
                                "state": "California",
                            },
                            "ip": "1.2.3.4",
                            "version": "V4",
                        },
                    ],
                },
                "securitycontext": {
                    "asNumber": 11351,
                    "asOrg": "charter communications inc",
                    "domain": "rr.com",
                    "isProxy": False,
                    "isp": "charter communications inc",
                },
                "severity": "WARN",
                "target": [
                    {"alternateId": "App (123)", "displayName": "App (123)", "id": "12345", "type": "AppInstance"},
                ],
                "transaction": {"detail": {}, "id": "aaa-bbb-123", "type": "WEB"},
                "uuid": "aa-11-22-33-44-bb",
                "version": "0",
            },
        ),
        RuleTest(
            name="system.operation.ratelimit.violation - unknown altid",
            expected_result=True,
            log={
                "actor": {"alternateId": "unknown", "displayName": "Homer Simpson", "id": "00abc456", "type": "User"},
                "authenticationcontext": {"authenticationStep": 0, "externalSessionId": "abc12345"},
                "client": {
                    "device": "Unknown",
                    "ipAddress": "1.2.3.4",
                    "userAgent": {"browser": "UNKNOWN", "os": "Unknown", "rawUserAgent": "Chrome"},
                    "zone": "null",
                },
                "debugcontext": {
                    "debugData": {
                        "authnRequestId": "ABCDFDE",
                        "dtHash": "adfalsjflasfjsdfd",
                        "operationRateLimitScopeType": "User",
                        "operationRateLimitSecondsToReset": "10",
                        "operationRateLimitSubtype": "Authenticated user",
                        "operationRateLimitThreshold": "40",
                        "operationRateLimitTimeSpan": "10",
                        "operationRateLimitTimeUnit": "SECONDS",
                        "operationRateLimitType": "Web request",
                        "requestId": "asfsagadffdaf",
                        "requestUri": "/app/google/",
                        "url": "/app/google/",
                    },
                },
                "displaymessage": "Operation rate limit violation",
                "eventtype": "system.operation.rate_limit.violation",
                "outcome": {"reason": "Too many requests attempted by an individual user", "result": "DENY"},
                "published": "2022-08-29 16:07:26.592",
                "request": {"ipChain": [{"ip": "1.2.3.4", "version": "V4"}]},
                "securitycontext": {},
                "severity": "WARN",
                "target": [{"id": "/app/{app}/{key}/", "type": "URL Pattern"}],
                "transaction": {"detail": {}, "id": "YABCDE", "type": "WEB"},
                "uuid": "asdfdashh",
                "version": "0",
            },
        ),
    ]



"""
Global helpers for Auth0 detections
"""


def auth0_alert_context(event) -> dict:
    a_c = {}
    a_c["actor"] = event.deep_get(
        "data",
        "details",
        "request",
        "auth",
        "user",
        default="<NO_ACTOR_FOUND>",
    )
    a_c["action"] = event.deep_get("data", "description", default="<NO_ACTION_FOUND>")
    return a_c


def is_auth0_config_event(event):
    channel = event.deep_get("data", "details", "request", "channel", default="")
    return channel == "https://manage.auth0.com/"



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.gcp import gcp_alert_context


@panther_managed
class GCPPrivilegedOperation(Rule):
    id = "GCP.Privileged.Operation-prototype"
    default_description = "Detects privileged operations in GCP that could be part of a privilege escalation attempt, especially when following tag binding creation.\n"
    display_name = "GCP Privileged Operation"
    log_types = [LogType.GCP_AUDIT_LOG]
    create_alert = False
    default_runbook = "Check if the user has legitimate business need for this privileged operation. If unauthorized, revoke any recently created tag bindings and review IAM policies.\n"
    default_severity = Severity.INFO
    tags = ["attack.privilege_escalation", "attack.t1548", "gcp", "iam", "tagbinding"]
    PRIVILEGED_OPERATIONS = [
        "iam.serviceAccounts.getAccessToken",
        "orgpolicy.policy.set",
        "storage.hmacKeys.create",
        "serviceusage.apiKeys.create",
        "serviceusage.apiKeys.list",
    ]

    def rule(self, event):
        method_name = event.deep_get("protoPayload", "methodName", default="")
        return (
            method_name.endswith("setIamPolicy")
            or method_name.endswith("setIamPermissions")
            or method_name in self.PRIVILEGED_OPERATIONS
        )

    def title(self, event):
        principal = event.deep_get("protoPayload", "authenticationInfo", "principalEmail", default="<UNKNOWN>")
        method = event.deep_get("protoPayload", "methodName", default="<UNKNOWN>")
        return f"GCP Privileged Operation by {principal} - {method}"

    def alert_context(self, event):
        return gcp_alert_context(event)

    tests = [
        RuleTest(
            name="Privileged Operation",
            expected_result=True,
            log={
                "protoPayload": {
                    "methodName": "compute.instances.setIamPolicy",
                    "authenticationInfo": {"principalEmail": "test@example.com"},
                    "resourceName": "projects/test-project",
                },
                "resource": {"labels": {"project_id": "test-project"}},
                "timestamp": "2024-01-01T00:00:00Z",
            },
        ),
        RuleTest(
            name="Normal Operation",
            expected_result=False,
            log={
                "protoPayload": {
                    "methodName": "compute.instances.list",
                    "authenticationInfo": {"principalEmail": "test@example.com"},
                    "resourceName": "projects/test-project",
                },
                "resource": {"labels": {"project_id": "test-project"}},
                "timestamp": "2024-01-01T00:00:00Z",
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed


@panther_managed
class AxoniusAddExternalUser(Rule):
    id = "Axonius.AddExternalUser-prototype"
    display_name = "Axonius External User Added"
    log_types = [LogType.AXONIUS_ACTIVITY]
    tags = ["Axonius"]
    default_severity = Severity.LOW
    default_description = "Detects when an external user is added in Axonius"
    default_runbook = "Review if the external user is authorized"

    def rule(self, event):
        action = event.deep_get("event", "action", default="")
        category = event.deep_get("event", "category", default="")
        if action == "AuditAction.AddExternalUser" and category == "AuditCategory.UserManagement":
            return True
        return False

    def title(self, event):
        username = event.deep_get("event", "params", "user_name", default="")
        source = event.deep_get("event", "params", "source", default="")
        return f"[Axonius] External User {username} added from {source}"

    def alert_context(self, event):
        username = event.deep_get("event", "params", "user_name", default="")
        source = event.deep_get("event", "params", "source", default="")
        context = {"username": username, "source": source}
        return context

    tests = [
        RuleTest(
            name="External User Created",
            expected_result=True,
            log={
                "event": {
                    "action": "AuditAction.AddExternalUser",
                    "category": "AuditCategory.UserManagement",
                    "params": {"source": "SAM-XYX", "user_name": "external-user@axonius.com"},
                    "type": "info",
                },
                "source": "axonius",
                "time": 1747795144.253623,
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.gcp import gcp_alert_context


@panther_managed
class GCPK8SServiceTypeNodePortDeployed(Rule):
    id = "GCP.K8S.Service.Type.NodePort.Deployed-prototype"
    display_name = "GCP K8S Service Type NodePort Deployed"
    log_types = [LogType.GCP_AUDIT_LOG]
    default_severity = Severity.HIGH
    default_description = "This detection monitors for any kubernetes service deployed with type node port. A Node Port service allows an attacker to expose a set of pods hosting the service to the internet by opening their port and redirecting traffic here. This can be used to bypass network controls and intercept traffic, creating a direct line to the outside network.\n"
    default_runbook = "Investigate the reason of creating NodePort service. Advise that it is discouraged practice. Create ticket if appropriate.\n"
    default_reference = "https://kubernetes.io/docs/tutorials/kubernetes-basics/expose/expose-intro/"
    tags = ["Exploit Public-Facing Application", "Initial Access"]
    reports = {"MITRE ATT&CK": ["TA0001:T1190"]}

    def rule(self, event):
        if event.deep_get("protoPayload", "response", "status") == "Failure":
            return False
        if event.deep_get("protoPayload", "methodName") != "io.k8s.core.v1.services.create":
            return False
        if event.deep_get("protoPayload", "request", "spec", "type") != "NodePort":
            return False
        authorization_info = event.deep_walk("protoPayload", "authorizationInfo")
        if not authorization_info:
            return False
        for auth in authorization_info:
            if auth.get("permission") == "io.k8s.core.v1.services.create" and auth.get("granted") is True:
                return True
        return False

    def title(self, event):
        actor = event.deep_get("protoPayload", "authenticationInfo", "principalEmail", default="<ACTOR_NOT_FOUND>")
        project_id = event.deep_get("resource", "labels", "project_id", default="<PROJECT_NOT_FOUND>")
        return f"[GCP]: [{actor}] created NodePort service in project [{project_id}]"

    def alert_context(self, event):
        context = gcp_alert_context(event)
        request_spec = event.deep_walk("protoPayload", "request", "spec")
        context["request_spec"] = request_spec
        return context

    tests = [
        RuleTest(
            name="Service Created",
            expected_result=True,
            log={
                "logName": "projects/some-project/logs/cloudaudit.googleapis.com%2Factivity",
                "protoPayload": {
                    "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                    "authenticationInfo": {"principalEmail": "some.user@company.com"},
                    "authorizationInfo": [
                        {
                            "granted": True,
                            "permission": "io.k8s.core.v1.services.create",
                            "resource": "core/v1/namespaces/default/services/test-ns",
                        },
                    ],
                    "methodName": "io.k8s.core.v1.services.create",
                    "request": {
                        "@type": "core.k8s.io/v1.Service",
                        "apiVersion": "v1",
                        "kind": "Service",
                        "spec": {
                            "ports": [{"name": "5678-8080", "port": 5678, "protocol": "TCP", "targetPort": 8080}],
                            "type": "NodePort",
                        },
                    },
                    "requestMetadata": {
                        "callerIP": "1.2.3.4",
                        "callerSuppliedUserAgent": "kubectl/v1.28.2 (darwin/amd64) kubernetes/89a4ea3",
                    },
                    "resourceName": "core/v1/namespaces/default/services/test-ns",
                    "response": {
                        "@type": "core.k8s.io/v1.Service",
                        "apiVersion": "v1",
                        "kind": "Service",
                        "metadata": {
                            "creationTimestamp": "2024-02-19T12:02:21Z",
                            "name": "test-ns",
                            "namespace": "default",
                            "resourceVersion": "15036073",
                            "uid": "28758fe1-534a-4705-bcc2-12eeac6f11a4",
                        },
                        "spec": {
                            "clusterIP": "2.3.4.5",
                            "clusterIPs": ["2.3.4.5"],
                            "ports": [
                                {
                                    "name": "5678-8080",
                                    "nodePort": 32361,
                                    "port": 5678,
                                    "protocol": "TCP",
                                    "targetPort": 8080,
                                },
                            ],
                            "type": "NodePort",
                        },
                    },
                    "serviceName": "k8s.io",
                    "status": {},
                },
                "receiveTimestamp": "2024-02-19 12:02:39.542633547",
                "resource": {
                    "labels": {
                        "cluster_name": "some-project-cluster",
                        "location": "us-west1",
                        "project_id": "some-project",
                    },
                    "type": "k8s_cluster",
                },
                "timestamp": "2024-02-19 12:02:22.057586000",
            },
        ),
        RuleTest(
            name="Error Creating Service",
            expected_result=False,
            log={
                "logName": "projects/some-project/logs/cloudaudit.googleapis.com%2Factivity",
                "protoPayload": {
                    "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                    "authenticationInfo": {"principalEmail": "some.user@company.com"},
                    "authorizationInfo": [
                        {
                            "granted": True,
                            "permission": "io.k8s.core.v1.services.create",
                            "resource": "core/v1/namespaces/default/services/test-ns",
                        },
                    ],
                    "methodName": "io.k8s.core.v1.services.create",
                    "request": {
                        "@type": "core.k8s.io/v1.Service",
                        "apiVersion": "v1",
                        "kind": "Service",
                        "spec": {
                            "ports": [{"name": "5678-8080", "port": 5678, "protocol": "TCP", "targetPort": 8080}],
                            "type": "NodePort",
                        },
                    },
                    "requestMetadata": {
                        "callerIP": "1.2.3.4",
                        "callerSuppliedUserAgent": "kubectl/v1.28.2 (darwin/amd64) kubernetes/89a4ea3",
                    },
                    "resourceName": "core/v1/namespaces/default/services/test-ns",
                    "response": {
                        "@type": "core.k8s.io/v1.Status",
                        "apiVersion": "v1",
                        "code": 409,
                        "details": {"kind": "services", "name": "test-ns"},
                        "kind": "Status",
                        "message": 'services "test-ns" already exists',
                        "metadata": {},
                        "reason": "AlreadyExists",
                        "status": "Failure",
                    },
                    "serviceName": "k8s.io",
                    "status": {"code": 10, "message": 'services "test-ns" already exists'},
                },
                "receiveTimestamp": "2024-02-20 13:47:46.955496128",
                "resource": {
                    "labels": {
                        "cluster_name": "some-project-cluster",
                        "location": "us-west1",
                        "project_id": "some-project",
                    },
                    "type": "k8s_cluster",
                },
                "timestamp": "2024-02-20 13:47:43.126037000",
            },
        ),
        RuleTest(
            name="No Permission Granted",
            expected_result=False,
            log={
                "logName": "projects/some-project/logs/cloudaudit.googleapis.com%2Factivity",
                "protoPayload": {
                    "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                    "authenticationInfo": {"principalEmail": "some.user@company.com"},
                    "authorizationInfo": [
                        {
                            "granted": False,
                            "permission": "io.k8s.core.v1.services.create",
                            "resource": "core/v1/namespaces/default/services/test-ns",
                        },
                    ],
                    "methodName": "io.k8s.core.v1.services.create",
                    "request": {
                        "@type": "core.k8s.io/v1.Service",
                        "apiVersion": "v1",
                        "kind": "Service",
                        "spec": {
                            "ports": [{"name": "5678-8080", "port": 5678, "protocol": "TCP", "targetPort": 8080}],
                            "type": "NodePort",
                        },
                    },
                    "requestMetadata": {
                        "callerIP": "1.2.3.4",
                        "callerSuppliedUserAgent": "kubectl/v1.28.2 (darwin/amd64) kubernetes/89a4ea3",
                    },
                    "resourceName": "core/v1/namespaces/default/services/test-ns",
                    "spec": {
                        "clusterIP": "2.3.4.5",
                        "clusterIPs": ["2.3.4.5"],
                        "externalTrafficPolicy": "Cluster",
                        "internalTrafficPolicy": "Cluster",
                        "ipFamilies": ["IPv4"],
                        "ipFamilyPolicy": "SingleStack",
                        "ports": [
                            {
                                "name": "5678-8080",
                                "nodePort": 32361,
                                "port": 5678,
                                "protocol": "TCP",
                                "targetPort": 8080,
                            },
                        ],
                        "type": "NodePort",
                    },
                },
                "serviceName": "k8s.io",
                "status": {},
                "receiveTimestamp": "2024-02-19 12:02:39.542633547",
                "resource": {
                    "labels": {
                        "cluster_name": "some-project-cluster",
                        "location": "us-west1",
                        "project_id": "some-project",
                    },
                    "type": "k8s_cluster",
                },
                "timestamp": "2024-02-19 12:02:22.057586000",
            },
        ),
    ]



from ipaddress import ip_network

from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.aws import aws_rule_context


@panther_managed
class AWSVPCSSHAllowedSignal(Rule):
    id = "AWS.VPC.SSHAllowedSignal-prototype"
    display_name = "Signal - VPC Flow Logs Allowed SSH"
    create_alert = False
    log_types = [LogType.AWS_VPC_FLOW]
    tags = ["AWS", "Signal"]
    reports = {"MITRE ATT&CK": ["TA0008:T1021.004"]}
    default_severity = Severity.INFO
    default_description = (
        "VPC Flow Logs observed inbound traffic on SSH port. This rule is a signal to be used in correlation rules.\n"
    )

    def rule(self, event):
        # Defaults to True (no alert) if 'dstport' is not present
        if event.udm("destination_port") != 22 or event.get("action") != "ACCEPT":
            return False
        # Only monitor for traffic coming from non-private IP space
        #
        # Defaults to True (no alert) if 'srcaddr' key is not present
        source_ip = event.udm("source_ip") or "0.0.0.0/32"
        if not ip_network(source_ip).is_global:
            return False
        # Alert if the traffic is destined for internal IP addresses
        #
        # Defaults to False(no alert) if 'dstaddr' key is not present
        destination_ip = event.udm("destination_ip") or "1.0.0.0/32"
        return not ip_network(destination_ip).is_global

    def alert_context(self, event):
        return aws_rule_context(event)

    tests = [
        RuleTest(
            name="Public to Private IP on SSH Allowed",
            expected_result=True,
            log={
                "dstPort": 22,
                "dstAddr": "10.0.0.1",
                "srcAddr": "1.1.1.1",
                "instanceId": "i-0d4c7318592c6a2c7",
                "action": "ACCEPT",
                "p_log_type": "AWS.VPCFlow",
            },
        ),
        RuleTest(
            name="Public to Private IP on non-SSH",
            expected_result=False,
            log={
                "dstPort": 443,
                "dstAddr": "10.0.0.1",
                "srcAddr": "1.1.1.1",
                "instanceId": "i-0d4c7318592c6a2c7",
                "action": "ACCEPT",
                "p_log_type": "AWS.VPCFlow",
            },
        ),
        RuleTest(
            name="Private to Private IP on SSH",
            expected_result=False,
            log={
                "dstPort": 22,
                "dstAddr": "10.0.0.1",
                "srcAddr": "10.10.10.1",
                "instanceId": "i-0d4c7318592c6a2c7",
                "action": "ACCEPT",
                "p_log_type": "AWS.VPCFlow",
            },
        ),
        RuleTest(
            name="Public to Private IP on SSH Denied",
            expected_result=False,
            log={
                "dstPort": 22,
                "dstAddr": "10.0.0.1",
                "srcAddr": "1.1.1.1",
                "instanceId": "i-0d4c7318592c6a2c7",
                "action": "REJECT",
                "p_log_type": "AWS.VPCFlow",
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed


@panther_managed
class GCPIAMCustomRoleChanges(Rule):
    id = "GCP.IAM.CustomRoleChanges-prototype"
    display_name = "GCP IAM Role Has Changed"
    dedup_period_minutes = 1440
    log_types = [LogType.GCP_AUDIT_LOG]
    tags = ["GCP", "Identity & Access Management", "Privilege Escalation:Valid Accounts"]
    reports = {"CIS": ["2.6"], "MITRE ATT&CK": ["TA0004:T1078"]}
    default_severity = Severity.INFO
    default_description = "A custom role has been created, deleted, or updated."
    default_runbook = "No action needed, informational"
    default_reference = "https://cloud.google.com/iam/docs/creating-custom-roles"
    summary_attributes = ["severity", "p_any_ip_addresses", "p_any_domain_names"]
    ROLE_METHODS = {
        "google.iam.admin.v1.CreateRole",
        "google.iam.admin.v1.DeleteRole",
        "google.iam.admin.v1.UpdateRole",
    }

    def rule(self, event):
        return (
            event.deep_get("resource", "type") == "iam_role"
            and event.deep_get("protoPayload", "methodName") in self.ROLE_METHODS
        )

    def dedup(self, event):
        return event.deep_get("resource", "labels", "project_id", default="<UNKNOWN_PROJECT>")

    tests = [
        RuleTest(
            name="Custom Role Created",
            expected_result=True,
            log={
                "protoPayload": {
                    "@type": "type.googleapis.com/google.cloud.audit.AuditLog",
                    "status": {},
                    "authenticationInfo": {
                        "principalEmail": "user.name@runpanther.io",
                        "principalSubject": "user:user.name@runpanther.io",
                    },
                    "requestMetadata": {
                        "callerIp": "136.24.229.58",
                        "callerSuppliedUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_4) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/81.0.4044.138 Safari/537.36,gzip(gfe)",
                        "requestAttributes": {"time": "2020-05-15T04:11:28.411897632Z", "auth": {}},
                        "destinationAttributes": {},
                    },
                    "serviceName": "iam.googleapis.com",
                    "methodName": "google.iam.admin.v1.CreateRole",
                    "authorizationInfo": [
                        {
                            "resource": "projects/western-verve-123456",
                            "permission": "iam.roles.create",
                            "granted": True,
                            "resourceAttributes": {},
                        },
                    ],
                    "resourceName": "projects/western-verve-123456/roles/CustomRole",
                    "serviceData": {
                        "@type": "type.googleapis.com/google.iam.admin.v1.AuditData",
                        "permissionDelta": {
                            "addedPermissions": [
                                "apigee.apiproducts.create",
                                "apigee.apiproducts.delete",
                                "apigee.apiproducts.get",
                            ],
                        },
                    },
                    "request": {
                        "parent": "projects/western-verve-123456",
                        "role": {
                            "stage": 1,
                            "included_permissions": [
                                "apigee.apiproducts.create",
                                "apigee.apiproducts.delete",
                                "apigee.apiproducts.get",
                            ],
                            "title": "Jack's custom role",
                            "description": "Created on: 2020-05-14",
                        },
                        "@type": "type.googleapis.com/google.iam.admin.v1.CreateRoleRequest",
                        "role_id": "CustomRole",
                    },
                    "response": {
                        "name": "projects/western-verve-123456/roles/CustomRole",
                        "group_title": "Custom",
                        "title": "Jack's custom role",
                        "included_permissions": [
                            "apigee.apiproducts.create",
                            "apigee.apiproducts.delete",
                            "apigee.apiproducts.get",
                        ],
                        "@type": "type.googleapis.com/google.iam.admin.v1.Role",
                        "description": "Created on: 2020-05-14",
                        "group_name": "custom",
                        "etag": "BwWlqAHm9IY=",
                        "stage": 1,
                    },
                },
                "insertId": "y4nffme2rory",
                "resource": {
                    "type": "iam_role",
                    "labels": {
                        "project_id": "western-verve-123456",
                        "role_name": "projects/western-verve-123456/roles/CustomRole",
                    },
                },
                "timestamp": "2020-05-15T04:11:28.224558457Z",
                "severity": "NOTICE",
                "logName": "projects/western-verve-123456/logs/cloudaudit.googleapis.com%2Factivity",
                "receiveTimestamp": "2020-05-15T04:11:29.472913078Z",
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.notion import notion_alert_context


@panther_managed
class NotionLogin(Rule):
    id = "Notion.Login-prototype"
    display_name = "Signal - Notion Login"
    create_alert = False
    log_types = [LogType.NOTION_AUDIT_LOGS]
    tags = ["Notion", "Identity & Access Management"]
    default_severity = Severity.INFO
    default_description = "A Notion User logged in."
    default_reference = "https://www.notion.so/help/account-settings"

    def rule(self, event):
        if event.deep_walk("event", "type") == "user.login":
            return True
        return False

    def title(self, event):
        user_email = event.deep_walk("event", "actor", "person", "email", default="UNKNOWN EMAIL")
        return f"Notion User [{user_email}] logged in."

    def alert_context(self, event):
        context = notion_alert_context(event)
        context["login_timestamp"] = event.get("p_event_time")
        context["actor_id"] = event.deep_walk("event", "actor", "id")
        return context

    tests = [
        RuleTest(
            name="Login event",
            expected_result=True,
            log={
                "event": {
                    "actor": {
                        "id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
                        "object": "user",
                        "person": {"email": "aragorn.elessar@lotr.com"},
                        "type": "person",
                    },
                    "details": {"authType": "email"},
                    "id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
                    "ip_address": "192.168.100.100",
                    "platform": "web",
                    "timestamp": "2023-06-12 21:40:28.690000000",
                    "type": "user.login",
                    "workspace_id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
                },
                "p_event_time": "2023-06-12 21:40:28.690000000",
                "p_log_type": "Notion.AuditLogs",
                "p_parse_time": "2023-06-12 22:53:51.602223297",
                "p_row_id": "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
                "p_schema_version": 0,
                "p_source_id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
                "p_source_label": "Notion Logs",
            },
        ),
        RuleTest(
            name="Not login event",
            expected_result=False,
            log={
                "event": {
                    "actor": {
                        "id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
                        "object": "user",
                        "person": {"email": "aragorn.elessar@lotr.com"},
                        "type": "person",
                    },
                    "details": {"authType": "email"},
                    "id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
                    "ip_address": "192.168.100.100",
                    "platform": "web",
                    "timestamp": "2023-06-12 21:40:28.690000000",
                    "type": "user.settings.login_method.email_updated",
                    "workspace_id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
                },
                "p_event_time": "2023-06-12 21:40:28.690000000",
                "p_log_type": "Notion.AuditLogs",
                "p_parse_time": "2023-06-12 22:53:51.602223297",
                "p_row_id": "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
                "p_schema_version": 0,
                "p_source_id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
                "p_source_label": "Notion Logs",
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.aws import aws_cloudtrail_success


@panther_managed
class AWSRootActivity(Rule):
    id = "AWS.Root.Activity-prototype"
    display_name = "Root Account Activity"
    log_types = [LogType.AWS_CLOUDTRAIL]
    tags = ["AWS", "Identity & Access Management", "DemoThreatHunting", "Privilege Escalation:Valid Accounts"]
    reports = {"CIS": ["3.3"], "MITRE ATT&CK": ["TA0004:T1078"]}
    default_severity = Severity.HIGH
    default_description = "Root account activity was detected.\n"
    default_runbook = "Investigate the usage of the root account. If this root activity was not authorized, immediately change the root credentials and investigate what actions the root account took.\n"
    default_reference = "https://docs.aws.amazon.com/IAM/latest/UserGuide/id_root-user.html"
    summary_attributes = [
        "awsRegion",
        "eventName",
        "eventSource",
        "userAgent",
        "p_any_aws_account_ids",
        "p_any_aws_arns",
        "p_any_ip_addresses",
    ]
    EVENT_ALLOW_LIST = {"CreateServiceLinkedRole"}

    def rule(self, event):
        return (
            event.deep_get("userIdentity", "type") == "Root"
            and aws_cloudtrail_success(event)
            and (event.deep_get("userIdentity", "invokedBy") is None)
            and (event.get("eventType") != "AwsServiceEvent")
            and (event.get("eventName") not in self.EVENT_ALLOW_LIST)
        )

    def dedup(self, event):
        return (
            event.get("sourceIPAddress", "<UNKNOWN_IP>")
            + ":"
            + event.get("recipientAccountId")
            + ":"
            + str(event.get("readOnly"))
        )

    def title(self, event):
        return f"AWS root user activity [{event.get('eventName')}] in account [{event.get('recipientAccountId')}]"

    def alert_context(self, event):
        return {
            "sourceIPAddress": event.get("sourceIPAddress"),
            "userIdentityAccountId": event.deep_get("userIdentity", "accountId"),
            "userIdentityArn": event.deep_get("userIdentity", "arn"),
            "eventTime": event.get("eventTime"),
            "mfaUsed": event.deep_get("additionalEventData", "MFAUsed"),
        }

    def severity(self, event):
        if event.get("readOnly"):
            return "LOW"
        return "HIGH"

    tests = [
        RuleTest(
            name="Root Activity - CreateServiceLinkedRole",
            expected_result=False,
            log={
                "eventVersion": "1.05",
                "userIdentity": {
                    "type": "Root",
                    "principalId": "123456789012",
                    "arn": "arn:aws:iam::123456789012:root",
                    "accountId": "123456789012",
                    "accessKeyId": "1",
                    "sessionContext": {
                        "attributes": {"mfaAuthenticated": "false", "creationDate": "2019-01-01T00:00:00Z"},
                    },
                },
                "eventTime": "2019-01-01T00:00:00Z",
                "eventSource": "s3.amazonaws.com",
                "eventName": "CreateServiceLinkedRole",
                "awsRegion": "us-west-2",
                "sourceIPAddress": "111.111.111.111",
                "userAgent": "Mozilla",
                "requestParameters": {
                    "bucketName": "bucket",
                    "versioning": [""],
                    "host": ["bucket.s3.us-west-2.amazonaws.com"],
                    "VersioningConfiguration": {
                        "Status": "Enabled",
                        "xmlns": "http://s3.amazonaws.com/doc/2006-03-01/",
                        "MfaDelete": "Enabled",
                    },
                },
                "responseElements": None,
                "additionalEventData": {
                    "SignatureVersion": "SigV4",
                    "CipherSuite": "ECDHE-RSA-AES128-GCM-SHA256",
                    "AuthenticationMethod": "AuthHeader",
                },
                "requestID": "1",
                "eventID": "1",
                "eventType": "AwsApiCall",
                "recipientAccountId": "123456789012",
            },
        ),
        RuleTest(
            name="Root Activity",
            expected_result=True,
            log={
                "eventVersion": "1.05",
                "userIdentity": {
                    "type": "Root",
                    "principalId": "123456789012",
                    "arn": "arn:aws:iam::123456789012:root",
                    "accountId": "123456789012",
                    "accessKeyId": "1",
                    "sessionContext": {
                        "attributes": {"mfaAuthenticated": "false", "creationDate": "2019-01-01T00:00:00Z"},
                    },
                },
                "eventTime": "2019-01-01T00:00:00Z",
                "eventSource": "s3.amazonaws.com",
                "eventName": "PutBucketVersioning",
                "awsRegion": "us-west-2",
                "sourceIPAddress": "111.111.111.111",
                "userAgent": "Mozilla",
                "requestParameters": {
                    "bucketName": "bucket",
                    "versioning": [""],
                    "host": ["bucket.s3.us-west-2.amazonaws.com"],
                    "VersioningConfiguration": {
                        "Status": "Enabled",
                        "xmlns": "http://s3.amazonaws.com/doc/2006-03-01/",
                        "MfaDelete": "Enabled",
                    },
                },
                "responseElements": None,
                "additionalEventData": {
                    "SignatureVersion": "SigV4",
                    "CipherSuite": "ECDHE-RSA-AES128-GCM-SHA256",
                    "AuthenticationMethod": "AuthHeader",
                },
                "requestID": "1",
                "eventID": "1",
                "eventType": "AwsApiCall",
                "recipientAccountId": "123456789012",
            },
        ),
        RuleTest(
            name="IAMUser Activity",
            expected_result=False,
            log={
                "eventVersion": "1.05",
                "userIdentity": {
                    "type": "IAMUser",
                    "principalId": "1111",
                    "arn": "arn:aws:iam::123456789012:user/tester",
                    "accountId": "123456789012",
                    "accessKeyId": "1",
                    "userName": "tester",
                    "sessionContext": {
                        "attributes": {"mfaAuthenticated": "false", "creationDate": "2019-01-01T00:00:00Z"},
                    },
                    "invokedBy": "signin.amazonaws.com",
                },
                "eventTime": "2019-01-01T00:00:00Z",
                "eventSource": "s3.amazonaws.com",
                "eventName": "GetBucketAcl",
                "awsRegion": "us-west-2",
                "sourceIPAddress": "111.111.111.111",
                "userAgent": "signin.amazonaws.com",
                "requestParameters": {
                    "host": ["bucket.s3.us-west-2.amazonaws.com"],
                    "bucketName": "bucket",
                    "acl": [""],
                },
                "responseElements": None,
                "additionalEventData": {
                    "SignatureVersion": "SigV4",
                    "CipherSuite": "ECDHE-RSA-AES128-SHA",
                    "AuthenticationMethod": "AuthHeader",
                    "vpcEndpointId": "vpce-1",
                },
                "requestID": "1",
                "eventID": "1",
                "eventType": "AwsApiCall",
                "recipientAccountId": "123456789012",
                "vpcEndpointId": "vpce-1",
            },
        ),
        RuleTest(
            name="Root User Failed Activity",
            expected_result=False,
            log={
                "awsRegion": "redacted",
                "errorMessage": "Not a valid response for the provided request id: aws_ccV60redacted",
                "eventID": "redacted",
                "eventName": "ExternalIdPDirectoryLogin",
                "eventSource": "signin.amazonaws.com",
                "eventTime": "redacted",
                "eventType": "AwsConsoleSignIn",
                "eventVersion": "1.05",
                "p_alert_creation_time": "redacted",
                "p_alert_id": "redacted",
                "p_alert_update_time": "redacted",
                "p_any_aws_account_ids": ["redacted"],
                "p_any_aws_arns": [],
                "p_any_ip_addresses": ["redacted"],
                "p_event_time": "redacted",
                "p_log_type": "AWS.CloudTrail",
                "p_parse_time": "redacted",
                "p_row_id": "redacted",
                "p_rule_error": None,
                "p_rule_id": "AWS.Root.Activity",
                "p_rule_reports": {"CIS": ["3.3"]},
                "p_rule_tags": ["AWS", "Identity & Access Management"],
                "p_source_id": "redacted",
                "p_source_label": "CloudTrail",
                "readOnly": False,
                "recipientAccountId": "redacted",
                "requestID": "redacted",
                "responseElements": {"ExternalIdPDirectoryLogin": "Failure"},
                "sourceIPAddress": "redacted",
                "userAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/84.0.4147.105 Safari/537.36",
                "userIdentity": {
                    "accessKeyId": "",
                    "accountId": "redacted",
                    "arn": "",
                    "principalId": "redacted",
                    "type": "Root",
                },
            },
        ),
        RuleTest(
            name="Successful Root Login",
            expected_result=True,
            log={
                "eventVersion": "1.05",
                "userIdentity": {
                    "type": "Root",
                    "principalId": "1111",
                    "arn": "arn:aws:iam::123456789012:root",
                    "accountId": "123456789012",
                    "userName": "root",
                },
                "eventTime": "2019-01-01T00:00:00Z",
                "eventSource": "signin.amazonaws.com",
                "eventName": "ConsoleLogin",
                "awsRegion": "us-east-1",
                "sourceIPAddress": "111.111.111.111",
                "userAgent": "Mozilla",
                "requestParameters": None,
                "responseElements": {"ConsoleLogin": "Success"},
                "additionalEventData": {
                    "LoginTo": "https://console.aws.amazon.com/console/",
                    "MobileVersion": "No",
                    "MFAUsed": "No",
                },
                "eventID": "1",
                "eventType": "AwsConsoleSignIn",
                "recipientAccountId": "123456789012",
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed


@panther_managed
class GCPPermissionsGrantedtoCreateorManageServiceAccountKey(Rule):
    default_description = "Permissions granted to impersonate a service account. This includes predefined service account IAM roles granted at the parent project, folder or organization-level."
    display_name = "GCP Permissions Granted to Create or Manage Service Account Key"
    default_reference = "https://cloud.google.com/iam/docs/keys-create-delete"
    default_severity = Severity.LOW
    log_types = [LogType.GCP_AUDIT_LOG]
    id = "GCP.Permissions.Granted.to.Create.or.Manage.Service.Account.Key-prototype"
    SERVICE_ACCOUNT_MANAGE_ROLES = ["roles/iam.serviceAccountTokenCreator", "roles/iam.serviceAccountUser"]

    def rule(self, event):
        if "SetIAMPolicy" in event.deep_get("protoPayload", "methodName", default=""):
            role = event.deep_walk(
                "protoPayload",
                "serviceData",
                "policyDelta",
                "bindingDeltas",
                "role",
                default="",
                return_val="last",
            )
            action = event.deep_walk(
                "protoPayload",
                "serviceData",
                "policyDelta",
                "bindingDeltas",
                "action",
                default="",
                return_val="last",
            )
            return role in self.SERVICE_ACCOUNT_MANAGE_ROLES and action == "ADD"
        return False

    def title(self, event):
        actor = event.deep_get("protoPayload", "authenticationInfo", "principalEmail", default="<ACTOR_NOT_FOUND>")
        target = event.deep_get("resource", "labels", "email_id") or event.deep_get(
            "resource",
            "labels",
            "project_id",
            default="<TARGET_NOT_FOUND>",
        )
        return f"GCP: [{actor}] granted permissions to create or manage service account keys to [{target}]"

    def alert_context(self, event):
        return {"resource": event.get("resource"), "serviceData": event.deep_get("protoPayload", "serviceData")}

    tests = [
        RuleTest(
            name="other event",
            expected_result=False,
            log={
                "insertid": "abcdefghijklmn",
                "logname": "projects/gcp-project1/logs/cloudaudit.googleapis.com%2Factivity",
                "operation": {
                    "id": "1234567890123-gcp-project1:abcdefghijklmnopqrstuvwz",
                    "last": True,
                    "producer": "bigquery.googleapis.com",
                },
                "p_any_emails": ["user@company.io"],
                "p_any_ip_addresses": ["1.2.3.4"],
                "p_event_time": "2023-03-28 18:37:06.079",
                "p_log_type": "GCP.AuditLog",
                "p_parse_time": "2023-03-28 18:38:14.478",
                "p_row_id": "06bf03d9d5dfbadba981899e1787bf05",
                "p_schema_version": 0,
                "p_source_id": "964c7894-9a0d-4ddf-864f-0193438221d6",
                "p_source_label": "gcp-logsource",
                "protoPayload": {
                    "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                    "authenticationInfo": {"principalEmail": "user@company.io"},
                    "authorizationInfo": [
                        {
                            "granted": True,
                            "permission": "bigquery.tables.delete",
                            "resource": "projects/gcp-project1/datasets/test1/tables/newtable",
                        },
                    ],
                    "metadata": {
                        "@type": "type.googleapis.com/google.cloud.audit.BigQueryAuditMetadata",
                        "methodName": "google.cloud.bigquery.v2.JobService.InsertJob",
                        "requestMetadata": {
                            "callerIP": "1.2.3.4",
                            "callerSuppliedUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/111.0.0.0 Safari/537.36,gzip(gfe),gzip(gfe)",
                        },
                        "resourceName": "projects/gcp-project1/datasets/test1/tables/newtable",
                        "serviceName": "bigquery.googleapis.com",
                        "status": {},
                        "tableDeletion": {
                            "jobName": "projects/gcp-project1/jobs/bquxjob_5e4a0679_18729a639d7",
                            "reason": "QUERY",
                        },
                    },
                    "receivetimestamp": "2023-03-28 18:37:06.745",
                    "resource": {
                        "labels": {"dataset_id": "test1", "project_id": "gcp-project1"},
                        "type": "bigquery_dataset",
                    },
                    "severity": "NOTICE",
                    "timestamp": "2023-03-28 18:37:06.079",
                },
            },
        ),
        RuleTest(
            name="service account match",
            expected_result=True,
            log={
                "insertId": "hhpfjvdgakc",
                "logName": "projects/gcp-project1/logs/cloudaudit.googleapis.com%2Factivity",
                "p_any_emails": ["user@company.io"],
                "p_any_ip_addresses": ["1.2.3.4"],
                "p_event_time": "2023-04-10 18:36:30.838",
                "p_log_type": "GCP.AuditLog",
                "p_parse_time": "2023-04-10 18:38:14.607",
                "p_row_id": "5286b52d4095c9f1b2e8eabe178f8203",
                "p_schema_version": 0,
                "p_source_id": "5b77391b-afad-46c7-8ddc-b8e21d4726b3",
                "p_source_label": "gcplogsource2",
                "protoPayload": {
                    "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                    "authenticationInfo": {
                        "principalEmail": "user@company.io",
                        "principalSubject": "user:user@company.io",
                    },
                    "authorizationInfo": [
                        {
                            "granted": True,
                            "permission": "iam.serviceAccounts.setIamPolicy",
                            "resource": "projects/-/serviceAccounts/105537103139416651075",
                            "resourceAttributes": {"name": "projects/-/serviceAccounts/105537103139416651075"},
                        },
                    ],
                    "methodName": "google.iam.admin.v1.SetIAMPolicy",
                    "request": {
                        "@type": "type.googleapis.com/google.iam.v1.SetIamPolicyRequest",
                        "policy": {
                            "bindings": [
                                {
                                    "members": ["serviceAccount:test-account3@gcp-project1.iam.gserviceaccount.com"],
                                    "role": "roles/iam.serviceAccountTokenCreator",
                                },
                                {
                                    "members": ["serviceAccount:test-account3@gcp-project1.iam.gserviceaccount.com"],
                                    "role": "roles/iam.serviceAccountUser",
                                },
                            ],
                            "etag": "ACAB",
                            "version": 3,
                        },
                        "resource": "projects/gcp-project1/serviceAccounts/105537103139416651075",
                    },
                    "requestMetadata": {
                        "callerIP": "1.2.3.4",
                        "callerSuppliedUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/111.0.0.0 Safari/537.36,gzip(gfe)",
                        "destinationAttributes": {},
                        "requestAttributes": {"auth": {}, "time": "2023-04-10T18:36:30.994141642Z"},
                    },
                    "resourceName": "projects/-/serviceAccounts/105537103139416651075",
                    "response": {
                        "@type": "type.googleapis.com/google.iam.v1.Policy",
                        "bindings": [
                            {
                                "members": ["serviceAccount:test-account3@gcp-project1.iam.gserviceaccount.com"],
                                "role": "roles/iam.serviceAccountTokenCreator",
                            },
                            {
                                "members": ["serviceAccount:test-account3@gcp-project1.iam.gserviceaccount.com"],
                                "role": "roles/iam.serviceAccountUser",
                            },
                        ],
                        "etag": "BwX4/6dQjX4=",
                        "version": 1,
                    },
                    "serviceData": {
                        "@type": "type.googleapis.com/google.iam.v1.logging.AuditData",
                        "policyDelta": {
                            "bindingDeltas": [
                                {
                                    "action": "ADD",
                                    "member": "serviceAccount:test-account3@gcp-project1.iam.gserviceaccount.com",
                                    "role": "roles/iam.serviceAccountTokenCreator",
                                },
                                {
                                    "action": "ADD",
                                    "member": "serviceAccount:test-account3@gcp-project1.iam.gserviceaccount.com",
                                    "role": "roles/iam.serviceAccountUser",
                                },
                            ],
                        },
                    },
                    "serviceName": "iam.googleapis.com",
                    "status": {},
                },
                "receiveTimestamp": "2023-04-10 18:36:32.268",
                "resource": {
                    "labels": {
                        "email_id": "test-account3@gcp-project1.iam.gserviceaccount.com",
                        "project_id": "gcp-project1",
                        "unique_id": "105537103139416651075",
                    },
                    "type": "service_account",
                },
                "severity": "NOTICE",
                "timestamp": "2023-04-10 18:36:30.838",
            },
        ),
    ]



from pypanther.rules.snyk.snyk_misc_settings import SnykMiscSettings as SnykMiscSettings
from pypanther.rules.snyk.snyk_org_settings import SnykOrgSettings as SnykOrgSettings
from pypanther.rules.snyk.snyk_ou_change import SnykOUChange as SnykOUChange
from pypanther.rules.snyk.snyk_project_settings import SnykProjectSettings as SnykProjectSettings
from pypanther.rules.snyk.snyk_role_change import SnykRoleChange as SnykRoleChange
from pypanther.rules.snyk.snyk_svcacct_change import SnykServiceAccountChange as SnykServiceAccountChange
from pypanther.rules.snyk.snyk_system_externalaccess import SnykSystemExternalAccess as SnykSystemExternalAccess
from pypanther.rules.snyk.snyk_system_policysetting import SnykSystemPolicySetting as SnykSystemPolicySetting
from pypanther.rules.snyk.snyk_system_sso import SnykSystemSSO as SnykSystemSSO
from pypanther.rules.snyk.snyk_user_mgmt import SnykUserManagement as SnykUserManagement



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.asana import asana_alert_context


@panther_managed
class AsanaWorkspaceNewAdmin(Rule):
    default_description = "Admin role was granted to the user who previously did not have admin permissions"
    display_name = "Asana Workspace New Admin"
    default_reference = "https://help.asana.com/hc/en-us/articles/14141552580635-Admin-and-super-admin-roles-in-Asana"
    default_severity = Severity.HIGH
    log_types = [LogType.ASANA_AUDIT]
    id = "Asana.Workspace.New.Admin-prototype"

    def rule(self, event):
        new = event.deep_get("details", "new_value", default="")
        old = event.deep_get("details", "old_value", default="")
        return all([event.get("event_type") == "user_workspace_admin_role_changed", "admin" in new, "admin" not in old])

    def title(self, event):
        a_c = asana_alert_context(event)
        w_s = event.deep_get("details", "group", "name", default="<WS_NAME_NOT_FOUND>")
        return (
            f"Asana user [{a_c.get('resource_name')}] was made an admin in workspace [{w_s}] by [{a_c.get('actor')}]."
        )

    def alert_context(self, event):
        return asana_alert_context(event)

    tests = [
        RuleTest(
            name="Team made public",
            expected_result=False,
            log={
                "actor": {
                    "actor_type": "user",
                    "email": "homer.simpson@panther.io",
                    "gid": "12345",
                    "name": "Homer Simpson",
                },
                "context": {
                    "client_ip_address": "12.12.12.12",
                    "context_type": "web",
                    "user_agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/108.0.0.0 Safari/537.36",
                },
                "created_at": "2022-12-16 19:35:21.026",
                "details": {"new_value": "public"},
                "event_category": "access_control",
                "event_type": "team_privacy_settings_changed",
                "gid": "12345",
                "resource": {"gid": "12345", "name": "Example Team Name", "resource_type": "team"},
                "p_log_type": "Asana.Audit",
            },
        ),
        RuleTest(
            name="New Workspace Admin",
            expected_result=True,
            log={
                "p_log_type": "Asana.Audit",
                "actor": {
                    "actor_type": "user",
                    "email": "homer.simpson@example.io",
                    "gid": "1234",
                    "name": "Homer Simpson",
                },
                "context": {
                    "client_ip_address": "12.12.12.12",
                    "context_type": "web",
                    "user_agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/108.0.0.0 Safari/537.36",
                },
                "created_at": "2022-12-16 19:32:22.377",
                "details": {
                    "group": {"gid": "1234", "name": "Workspace Name", "resource_type": "workspace"},
                    "new_value": "domain_admin",
                    "old_value": "member",
                },
                "event_category": "roles",
                "event_type": "user_workspace_admin_role_changed",
                "gid": "1234",
                "resource": {
                    "email": "target@example.io",
                    "gid": "1234",
                    "name": "Bart Simpson",
                    "resource_type": "user",
                },
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.aws import aws_cloudtrail_success, aws_rule_context


@panther_managed
class AWSS3DeleteObject(Rule):
    display_name = "AWS S3 Delete Object Detection"
    id = "AWS.S3.DeleteObject-prototype"
    threshold = 50
    default_severity = Severity.INFO
    log_types = [LogType.AWS_CLOUDTRAIL]
    tags = ["AWS", "S3", "CloudTrail", "Beta"]
    default_description = "This rule detects when many objects are deleted from an S3 bucket. Such actions can be indicative of unauthorized data deletion or other suspicious activities.\n"
    default_runbook = "Investigate the user and the actions performed on the S3 bucket to ensure they were authorized. Unauthorized deletions can lead to data loss. Steps to investigate: 1. Identify the user who performed the action. 2. Verify if the action was authorized. 3. Check for any other suspicious activities performed by the same user. 4. If unauthorized, take necessary actions to secure the S3 bucket and prevent further unauthorized access.\n"
    default_reference = "https://docs.aws.amazon.com/AmazonS3/latest/userguide/delete-objects.html"

    def rule(self, event):
        return (
            aws_cloudtrail_success(event)
            and event.get("eventSource") == "s3.amazonaws.com"
            and (event.get("eventName") == "DeleteObject")
        )

    def title(self, event):
        return f"[AWS.CloudTrail] User [{event.udm('actor_user')}] deleted many items from the [{event.deep_get('requestParameters', 'bucketName')}] bucket"

    def alert_context(self, event):
        return aws_rule_context(event)

    tests = [
        RuleTest(
            name="DeleteObject",
            expected_result=True,
            log={
                "eventVersion": "1.08",
                "userIdentity": {
                    "type": "IAMUser",
                    "principalId": "EXAMPLE",
                    "arn": "arn:aws:iam::123456789012:user/Alice",
                    "accountId": "123456789012",
                    "accessKeyId": "EXAMPLEKEY",
                    "userName": "Alice",
                },
                "eventTime": "2023-10-01T12:34:56Z",
                "eventSource": "s3.amazonaws.com",
                "eventName": "DeleteObject",
                "awsRegion": "us-east-1",
                "sourceIPAddress": "192.0.2.0",
                "userAgent": "aws-sdk-go/1.15.12 (go1.12.6; linux; amd64)",
                "requestParameters": {"bucketName": "example-bucket", "key": "example-object"},
                "responseElements": None,
                "additionalEventData": {
                    "SignatureVersion": "SigV4",
                    "CipherSuite": "ECDHE-RSA-AES128-GCM-SHA256",
                    "bytesTransferredIn": 0,
                    "bytesTransferredOut": 0,
                },
                "requestID": "EXAMPLE123456789",
                "eventID": "EXAMPLE-1234-5678-9012-EXAMPLE",
                "readOnly": False,
                "resources": [{"type": "AWS::S3::Object", "ARN": "arn:aws:s3:::example-bucket/example-object"}],
                "eventType": "AwsApiCall",
                "managementEvent": False,
                "recipientAccountId": "123456789012",
                "sharedEventID": "EXAMPLE-1234-5678-9012-EXAMPLE",
                "vpcEndpointId": "vpce-1a2b3c4d",
            },
        ),
        RuleTest(
            name="Other Event",
            expected_result=False,
            log={
                "eventVersion": "1.08",
                "userIdentity": {
                    "type": "IAMUser",
                    "principalId": "EXAMPLE",
                    "arn": "arn:aws:iam::123456789012:user/Bob",
                    "accountId": "123456789012",
                    "accessKeyId": "EXAMPLEKEY",
                    "userName": "Bob",
                },
                "eventTime": "2023-10-01T12:34:56Z",
                "eventSource": "s3.amazonaws.com",
                "eventName": "ListBucket",
                "awsRegion": "us-east-1",
                "sourceIPAddress": "192.0.2.0",
                "userAgent": "aws-sdk-go/1.15.12 (go1.12.6; linux; amd64)",
                "requestParameters": {"bucketName": "example-bucket"},
                "responseElements": None,
                "additionalEventData": {
                    "SignatureVersion": "SigV4",
                    "CipherSuite": "ECDHE-RSA-AES128-GCM-SHA256",
                    "bytesTransferredIn": 0,
                    "bytesTransferredOut": 0,
                },
                "requestID": "EXAMPLE123456789",
                "eventID": "EXAMPLE-1234-5678-9012-EXAMPLE",
                "readOnly": True,
                "resources": [{"type": "AWS::S3::Bucket", "ARN": "arn:aws:s3:::example-bucket"}],
                "eventType": "AwsApiCall",
                "managementEvent": False,
                "recipientAccountId": "123456789012",
                "sharedEventID": "EXAMPLE-1234-5678-9012-EXAMPLE",
                "vpcEndpointId": "vpce-1a2b3c4d",
            },
        ),
    ]



from pypanther.rules.gravitational_teleport.teleport_auth_errors import TeleportAuthErrors as TeleportAuthErrors
from pypanther.rules.gravitational_teleport.teleport_company_domain_login_without_saml import (
    TeleportCompanyDomainLoginWithoutSAML as TeleportCompanyDomainLoginWithoutSAML,
)
from pypanther.rules.gravitational_teleport.teleport_create_user_accounts import (
    TeleportCreateUserAccounts as TeleportCreateUserAccounts,
)
from pypanther.rules.gravitational_teleport.teleport_local_user_login_without_mfa import (
    TeleportLocalUserLoginWithoutMFA as TeleportLocalUserLoginWithoutMFA,
)
from pypanther.rules.gravitational_teleport.teleport_lock_created import TeleportLockCreated as TeleportLockCreated
from pypanther.rules.gravitational_teleport.teleport_long_lived_certs import (
    TeleportLongLivedCerts as TeleportLongLivedCerts,
)
from pypanther.rules.gravitational_teleport.teleport_network_scanning import (
    TeleportNetworkScanning as TeleportNetworkScanning,
)
from pypanther.rules.gravitational_teleport.teleport_role_created import TeleportRoleCreated as TeleportRoleCreated
from pypanther.rules.gravitational_teleport.teleport_root_login import TeleportRootLogin as TeleportRootLogin
from pypanther.rules.gravitational_teleport.teleport_saml_created import TeleportSAMLCreated as TeleportSAMLCreated
from pypanther.rules.gravitational_teleport.teleport_saml_login_not_company_domain import (
    TeleportSAMLLoginWithoutCompanyDomain as TeleportSAMLLoginWithoutCompanyDomain,
)
from pypanther.rules.gravitational_teleport.teleport_scheduled_jobs import (
    TeleportScheduledJobs as TeleportScheduledJobs,
)
from pypanther.rules.gravitational_teleport.teleport_suspicious_commands import (
    TeleportSuspiciousCommands as TeleportSuspiciousCommands,
)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.aws import aws_cloudtrail_success, aws_rule_context


@panther_managed
class AWSEC2NetworkACLModified(Rule):
    id = "AWS.EC2.NetworkACLModified-prototype"
    display_name = "EC2 Network ACL Modified"
    log_types = [LogType.AWS_CLOUDTRAIL]
    tags = ["AWS", "Security Control", "Defense Evasion:Impair Defenses"]
    reports = {"CIS": ["3.11"], "MITRE ATT&CK": ["TA0005:T1562"]}
    default_severity = Severity.INFO
    default_description = "An EC2 Network ACL was modified."
    default_runbook = "https://docs.runpanther.io/alert-runbooks/built-in-rules/aws-ec2-network-acl-modified"
    default_reference = "https://docs.aws.amazon.com/vpc/latest/userguide/vpc-network-acls.html#nacl-tasks"
    summary_attributes = ["eventName", "userAgent", "sourceIpAddress", "recipientAccountId", "p_any_aws_arns"]
    # API calls that are indicative of an EC2 Network ACL modification
    EC2_NACL_MODIFIED_EVENTS = {
        "CreateNetworkAcl",
        "CreateNetworkAclEntry",
        "DeleteNetworkAcl",
        "DeleteNetworkAclEntry",
        "ReplaceNetworkAclEntry",
        "ReplaceNetworkAclAssociation",
    }

    def rule(self, event):
        return aws_cloudtrail_success(event) and event.get("eventName") in self.EC2_NACL_MODIFIED_EVENTS

    def dedup(self, event):
        return event.get("recipientAccountId")

    def alert_context(self, event):
        return aws_rule_context(event)

    tests = [
        RuleTest(
            name="Network ACL Modified",
            expected_result=True,
            log={
                "eventVersion": "1.05",
                "userIdentity": {
                    "type": "AssumedRole",
                    "principalId": "1111:tester",
                    "arn": "arn:aws:sts::123456789012:assumed-role/tester",
                    "accountId": "123456789012",
                    "accessKeyId": "1",
                    "sessionContext": {
                        "attributes": {"mfaAuthenticated": "true", "creationDate": "2019-01-01T00:00:00Z"},
                        "sessionIssuer": {
                            "type": "Role",
                            "principalId": "1111",
                            "arn": "arn:aws:iam::123456789012:role/tester",
                            "accountId": "123456789012",
                            "userName": "tester",
                        },
                    },
                },
                "eventTime": "2019-01-01T00:00:00Z",
                "eventSource": "ec2.amazonaws.com",
                "eventName": "CreateNetworkAclEntry",
                "awsRegion": "us-west-2",
                "sourceIPAddress": "111.111.111.111",
                "userAgent": "console.ec2.amazonaws.com",
                "requestParameters": {
                    "networkAclId": "acl-1",
                    "ruleNumber": 500,
                    "egress": True,
                    "ruleAction": "allow",
                    "icmpTypeCode": {},
                    "portRange": {},
                    "aclProtocol": "-1",
                    "cidrBlock": "0.0.0.0/0",
                },
                "responseElements": {"requestID": "1", "_return": True},
                "requestID": "1",
                "eventID": "1",
                "eventType": "AwsApiCall",
                "recipientAccountId": "123456789012",
            },
        ),
        RuleTest(
            name="Network ACL Not Modified",
            expected_result=False,
            log={
                "eventVersion": "1.05",
                "userIdentity": {
                    "type": "AssumedRole",
                    "principalId": "1111:tester",
                    "arn": "arn:aws:sts::123456789012:assumed-role/tester",
                    "accountId": "123456789012",
                    "accessKeyId": "1",
                    "sessionContext": {
                        "sessionIssuer": {
                            "type": "Role",
                            "principalId": "1111",
                            "arn": "arn:aws:iam::123456789012:role/tester",
                            "accountId": "123456789012",
                            "userName": "tester",
                        },
                        "webIdFederationData": {},
                        "attributes": {"mfaAuthenticated": "false", "creationDate": "2019-01-01T00:00:00Z"},
                    },
                },
                "eventTime": "2019-01-01T00:00:00Z",
                "eventSource": "ec2.amazonaws.com",
                "eventName": "DescribeNetworkAcls",
                "awsRegion": "us-west-2",
                "sourceIPAddress": "111.111.111.111",
                "userAgent": "Mozilla",
                "requestParameters": {
                    "networkAclIdSet": {},
                    "filterSet": {"items": [{"name": "vpc-id", "valueSet": {"items": [{"value": "vpc-1"}]}}]},
                },
                "responseElements": None,
                "requestID": "1",
                "eventID": "1",
                "eventType": "AwsApiCall",
                "recipientAccountId": "123456789012",
            },
        ),
        RuleTest(
            name="Error Modifying Network ACL",
            expected_result=False,
            log={
                "errorCode": "InvalidCharacter",
                "eventVersion": "1.05",
                "userIdentity": {
                    "type": "AssumedRole",
                    "principalId": "1111:tester",
                    "arn": "arn:aws:sts::123456789012:assumed-role/tester",
                    "accountId": "123456789012",
                    "accessKeyId": "1",
                    "sessionContext": {
                        "attributes": {"mfaAuthenticated": "true", "creationDate": "2019-01-01T00:00:00Z"},
                        "sessionIssuer": {
                            "type": "Role",
                            "principalId": "1111",
                            "arn": "arn:aws:iam::123456789012:role/tester",
                            "accountId": "123456789012",
                            "userName": "tester",
                        },
                    },
                },
                "eventTime": "2019-01-01T00:00:00Z",
                "eventSource": "ec2.amazonaws.com",
                "eventName": "CreateNetworkAclEntry",
                "awsRegion": "us-west-2",
                "sourceIPAddress": "111.111.111.111",
                "userAgent": "console.ec2.amazonaws.com",
                "requestParameters": {
                    "networkAclId": "acl-1",
                    "ruleNumber": 500,
                    "egress": True,
                    "ruleAction": "allow",
                    "icmpTypeCode": {},
                    "portRange": {},
                    "aclProtocol": "-1",
                    "cidrBlock": "0.0.0.0/0",
                },
                "responseElements": {"requestID": "1", "_return": True},
                "requestID": "1",
                "eventID": "1",
                "eventType": "AwsApiCall",
                "recipientAccountId": "123456789012",
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed


@panther_managed
class SnowflakeStreamLoginWithoutMFA(Rule):
    id = "Snowflake.Stream.LoginWithoutMFA-prototype"
    display_name = "Snowflake Login Without MFA"
    enabled = False
    log_types = [LogType.SNOWFLAKE_LOGIN_HISTORY]
    default_severity = Severity.MEDIUM
    reports = {"MITRE ATT&CK": ["TA0005:T1556"]}
    default_description = "Detect Snowflake logins without multifactor authentication"
    dedup_period_minutes = 1440
    tags = ["Snowflake", "[MITRE] Defense Evasion", "[MITRE] Modify Authentication Process"]
    MFA_EXCEPTIONS = {"PANTHER_READONLY", "PANTHER_ADMIN", "PANTHERACCOUNTADMIN"}

    def rule(self, event):
        return all(
            (
                event.get("EVENT_TYPE") == "LOGIN",
                event.get("IS_SUCCESS") == "YES",
                event.get("FIRST_AUTHENTICATION_FACTOR") == "PASSWORD",
                not event.get("SECOND_AUTHENTICATION_FACTOR"),
                event.get("USER_NAME") not in self.MFA_EXCEPTIONS,
            ),
        )

    def title(self, event):
        source = event.get("p_source_label", "<UNKNOWN SOURCE>")
        user = event.get("USER_NAME", "<UNKNOWN USER>")
        return f"{source}: User {user} logged in without MFA"

    tests = [
        RuleTest(
            name="Login With MFA",
            expected_result=False,
            log={
                "p_event_time": "2024-10-08 14:38:46.061000000",
                "p_log_type": "Snowflake.LoginHistory",
                "p_source_label": "Snowflake Prod",
                "CLIENT_IP": "1.1.1.1",
                "EVENT_ID": "393754014361778",
                "EVENT_TIMESTAMP": "2024-10-08 14:38:46.061000000",
                "EVENT_TYPE": "LOGIN",
                "FIRST_AUTHENTICATION_FACTOR": "PASSWORD",
                "IS_SUCCESS": "YES",
                "RELATED_EVENT_ID": "0",
                "REPORTED_CLIENT_TYPE": "OTHER",
                "REPORTED_CLIENT_VERSION": "1.11.1",
                "SECOND_AUTHENTICATION_FACTOR": "OTP",
                "USER_NAME": "ckent@dailyplanet.org",
            },
        ),
        RuleTest(
            name="Login Without MFA",
            expected_result=True,
            log={
                "p_event_time": "2024-10-08 14:38:46.061000000",
                "p_log_type": "Snowflake.LoginHistory",
                "p_source_label": "Snowflake Prod",
                "CLIENT_IP": "1.2.3.4",
                "EVENT_ID": "393754014361778",
                "EVENT_TIMESTAMP": "2024-10-08 14:38:46.061000000",
                "EVENT_TYPE": "LOGIN",
                "FIRST_AUTHENTICATION_FACTOR": "PASSWORD",
                "IS_SUCCESS": "YES",
                "RELATED_EVENT_ID": "0",
                "REPORTED_CLIENT_TYPE": "OTHER",
                "REPORTED_CLIENT_VERSION": "1.11.1",
                "USER_NAME": "luthor@lexcorp.com",
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed


@panther_managed
class SublimeMessageFlagged(Rule):
    default_description = "Sublime flagged some messages as suspicious."
    display_name = "Sublime Flagged an Email"
    default_severity = Severity.HIGH
    log_types = [LogType.SUBLIME_MESSAGE_EVENT]
    id = "Sublime.Message.Flagged-prototype"

    def rule(self, event):
        return event.get("type") == "message.flagged"

    def alert_context(self, event):
        flagged_rules = event.deep_walk("data", "flagged_rules", "name", default=["<UNKNOWN_NAMES>"])
        return {"flagged_rules": flagged_rules}

    def title(self, event):
        rule_count = len(event.deep_get("data", "flagged_rules", default=[]))
        return f"Sublime flagged email message that matched {rule_count} Sublime rules"

    tests = [
        RuleTest(
            name="Message Flagged",
            expected_result=True,
            log={
                "p_source_file": {
                    "aws_s3_bucket": "audit.log.export",
                    "aws_s3_key": "sublime_platform_message_events/2024/09/24/164544Z-FPXIFG.json",
                },
                "p_any_sha256_hashes": ["fb8b46e3317ac7d5036c6b21517d363634293c6d4f6bf1b1e67548c80948a1c6"],
                "p_event_time": "2024-09-24 16:45:43.302769000",
                "p_log_type": "Sublime.MessageEvent",
                "p_parse_time": "2024-09-24 16:51:47.687095351",
                "p_row_id": "a23385494d57dfbbbdcbe4fa218101",
                "p_schema_version": 0,
                "p_source_id": "7e2a59aa-687e-430e-ae4a-81d3c0163f52",
                "p_source_label": "Sublime Real Logs",
                "created_at": "2024-09-24 16:45:43.302769000",
                "data": {
                    "flagged_rules": [
                        {
                            "id": "b0ab266f-8a12-4020-b165-e97bb1aacc42",
                            "name": "Credential phishing: Engaging language and other indicators (untrusted sender)",
                        },
                        {
                            "id": "a014f82e-f2d7-4058-adb1-36fc086de0b8",
                            "name": "Attachment: HTML smuggling with unescape",
                        },
                        {
                            "id": "e4866908-60fe-46f0-866e-84d412627006",
                            "name": "Headers: Zimbra mailer from a non-supported OS version",
                        },
                        {
                            "id": "5a9dc2cd-39f5-4814-95df-aa7614cc8bdd",
                            "name": "Impersonation: Human Resources with link or attachment and engaging language",
                        },
                        {
                            "id": "7988f1f5-5c95-42c2-9140-ead5a975918e",
                            "name": "Request for Quote or Purchase (RFQ|RFP) with HTML smuggling attachment",
                        },
                    ],
                    "message": {
                        "canonical_id": "fb8b46e3317ac7d5036c6b21517d363634293c6d4f6bf1b1e67548c80948a1c6",
                        "external_id": "b86b1e58-e9f8-4b55-8b54-1402f9f95e69",
                        "id": "019224ec-aba6-763d-bb2e-cd4cbd40a29f",
                        "mailbox": {"id": "624c8394-4fe2-4ba0-bd2b-86d2e503c614"},
                        "message_source_id": "91956379-c2f3-4c50-a410-3ba89fb8bc74",
                    },
                },
                "type": "message.flagged",
            },
        ),
    ]



import time

from panther_detection_helpers.caching import get_counter, increment_counter, reset_counter, set_key_expiration

from pypanther import LogType, Rule, RuleMock, RuleTest, Severity, panther_managed


@panther_managed
class AxoniusTooManyFailedLogins(Rule):
    id = "Axonius.TooManyFailedLogins-prototype"
    display_name = "Axonius login from Tor IP"
    log_types = [LogType.AXONIUS_ACTIVITY]
    tags = ["Axonius"]
    default_severity = Severity.MEDIUM
    default_description = "Detects an Axonius login from a malicious IP address"
    default_runbook = "Review the actions taken, check credentials/service user used and escalate."
    MAX_FAILS = 5
    TIME_WINDOW = 60 * 5
    CACHE_PREFIX = "axonius_too_many_failed_logins:"

    def rule(self, event):
        action = event.deep_get("event", "action", default="")
        status = event.deep_get("event", "params", "status", default="")
        username = event.deep_get("event", "user", default="")
        if status != "successful" and action == "AuditAction.LoginFrom":
            cache_key = f"{self.CACHE_PREFIX}{username}"
            count = int(get_counter(cache_key))
            if count > self.MAX_FAILS:
                reset_counter(cache_key)
                return True
            increment_counter(cache_key)
            set_key_expiration(cache_key, time.time() + self.TIME_WINDOW)
        return False

    def title(self, event):
        username = event.deep_get("event", "user", default="")
        return f"[Axonius] Too many failed Login from {username}"

    def alert_context(self, event):
        username = event.deep_get("event", "user", default="")
        ip_address = event.deep_get("event", "params", "ip", default="")
        action = event.deep_get("event", "action", default="")
        status = event.deep_get("event", "params", "status", default="")
        context = {"username": username, "ip_address": ip_address, "action": action, "status": status}
        return context

    tests = [
        RuleTest(
            name="Successful Login",
            expected_result=False,
            mocks=[
                RuleMock(object_name="get_counter", return_value="0"),
                RuleMock(object_name="increment_counter", return_value=True),
                RuleMock(object_name="set_key_expiration", return_value=True),
            ],
            log={
                "event": {
                    "action": "AuditAction.LoginFrom",
                    "category": "AuditCategory.UserSession",
                    "params": {"ip": "1.2.3.4", "status": "successful"},
                    "type": "user",
                    "user": "denethor@lotr.com",
                },
                "source": "axonius",
                "time": 1752690986.344609,
            },
        ),
        RuleTest(
            name="One Failed Login",
            expected_result=False,
            mocks=[
                RuleMock(object_name="get_counter", return_value="0"),
                RuleMock(object_name="increment_counter", return_value=True),
                RuleMock(object_name="set_key_expiration", return_value=True),
            ],
            log={
                "event": {
                    "action": "AuditAction.LoginFrom",
                    "category": "AuditCategory.UserSession",
                    "params": {"ip": "1.2.3.4", "status": "failure"},
                    "type": "user",
                    "user": "denethor@lotr.com",
                },
                "source": "axonius",
                "time": 1752690986.344609,
            },
        ),
        RuleTest(
            name="Too Many Failed Login",
            expected_result=True,
            mocks=[
                RuleMock(object_name="get_counter", return_value="6"),
                RuleMock(object_name="increment_counter", return_value=True),
                RuleMock(object_name="set_key_expiration", return_value=True),
                RuleMock(object_name="reset_counter", return_value=True),
            ],
            log={
                "event": {
                    "action": "AuditAction.LoginFrom",
                    "category": "AuditCategory.UserSession",
                    "params": {"ip": "172.7.1.1", "status": "failure"},
                    "type": "user",
                    "user": "testr@axonius.com",
                },
                "source": "axonius",
                "time": 1752690986.344609,
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.slack import slack_alert_context


@panther_managed
class SlackAuditLogsApplicationDoS(Rule):
    id = "Slack.AuditLogs.ApplicationDoS-prototype"
    display_name = "Slack Denial of Service"
    log_types = [LogType.SLACK_AUDIT_LOGS]
    tags = ["Slack", "Impact", "Endpoint Denial of Service", "Application Exhaustion Flood"]
    reports = {"MITRE ATT&CK": ["TA0040:T1499.003"]}
    default_severity = Severity.CRITICAL
    default_description = "Detects when slack admin invalidates user session(s). If it happens more than once in a 24 hour period it can lead to DoS"
    default_reference = "https://slack.com/intl/en-gb/help/articles/115005223763-Manage-session-duration-#pro-and-business+-subscriptions-2"
    dedup_period_minutes = 1440
    threshold = 60
    summary_attributes = ["action", "p_any_ip_addresses", "p_any_emails"]
    DENIAL_OF_SERVICE_ACTIONS = [
        "bulk_session_reset_by_admin",
        "user_session_invalidated",
        "user_session_reset_by_admin",
    ]

    def rule(self, event):
        # Only evaluate actions that could be used for a DoS
        if event.get("action") not in self.DENIAL_OF_SERVICE_ACTIONS:
            return False
        return True

    def dedup(self, event):
        return f"Slack.AuditLogs.ApplicationDoS{event.deep_get('entity', 'user', 'name')}"

    def alert_context(self, event):
        return slack_alert_context(event)

    tests = [
        RuleTest(
            name="User Session Reset",
            expected_result=True,
            log={
                "action": "user_session_reset_by_admin",
                "actor": {
                    "type": "user",
                    "user": {
                        "email": "user@example.com",
                        "id": "W012J3FEWAU",
                        "name": "primary-owner",
                        "team": "T01234N56GB",
                    },
                },
                "context": {
                    "ip_address": "1.2.3.4",
                    "location": {
                        "domain": "test-workspace-1",
                        "id": "T01234N56GB",
                        "name": "test-workspace-1",
                        "type": "workspace",
                    },
                    "ua": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/103.0.0.0 Safari/537.36",
                },
            },
        ),
        RuleTest(
            name="Other action",
            expected_result=False,
            log={
                "action": "organization_created",
                "actor": {
                    "type": "user",
                    "user": {
                        "email": "user@example.com",
                        "id": "A012B3CDEFG",
                        "name": "username",
                        "team": "T01234N56GB",
                    },
                },
                "context": {
                    "ip_address": "1.2.3.4",
                    "location": {
                        "domain": "test-workspace",
                        "id": "T01234N56GB",
                        "name": "test-workspace",
                        "type": "workspace",
                    },
                    "ua": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/103.0.0.0 Safari/537.36",
                },
            },
        ),
    ]



from pypanther.rules.zoom_operation.zoom_all_meetings_secured_with_one_option_disabled import (
    ZoomAllMeetingsSecuredWithOneOptionDisabled as ZoomAllMeetingsSecuredWithOneOptionDisabled,
)
from pypanther.rules.zoom_operation.zoom_automatic_sign_out_disabled import (
    ZoomAutomaticSignOutDisabled as ZoomAutomaticSignOutDisabled,
)
from pypanther.rules.zoom_operation.zoom_new_meeting_passcode_required_disabled import (
    ZoomNewMeetingPasscodeRequiredDisabled as ZoomNewMeetingPasscodeRequiredDisabled,
)
from pypanther.rules.zoom_operation.zoom_operation_passcode_disabled import ZoomPasscodeDisabled as ZoomPasscodeDisabled
from pypanther.rules.zoom_operation.zoom_sign_in_method_modified import (
    ZoomSignInMethodModified as ZoomSignInMethodModified,
)
from pypanther.rules.zoom_operation.zoom_sign_in_requirements_changed import (
    ZoomSignInRequirementsChanged as ZoomSignInRequirementsChanged,
)
from pypanther.rules.zoom_operation.zoom_two_factor_authentication_disabled import (
    ZoomTwoFactorAuthenticationDisabled as ZoomTwoFactorAuthenticationDisabled,
)
from pypanther.rules.zoom_operation.zoom_user_promoted_to_privileged_role import (
    ZoomUserPromotedtoPrivilegedRole as ZoomUserPromotedtoPrivilegedRole,
)



from pypanther.rules.netskope.netskope_admin_logged_out import (
    NetskopeAdminLoggedOutLoginFailures as NetskopeAdminLoggedOutLoginFailures,
)
from pypanther.rules.netskope.netskope_admin_user_change import NetskopeAdminUserChange as NetskopeAdminUserChange
from pypanther.rules.netskope.netskope_many_deletes import NetskopeManyDeletes as NetskopeManyDeletes
from pypanther.rules.netskope.netskope_personnel_action import (
    NetskopeNetskopePersonnelActivity as NetskopeNetskopePersonnelActivity,
)
from pypanther.rules.netskope.netskope_unauthorized_api_calls import (
    NetskopeUnauthorizedAPICalls as NetskopeUnauthorizedAPICalls,
)



from pypanther.rules.aws_vpc_flow.aws_dns_crypto_domain import AWSDNSCryptoDomain as AWSDNSCryptoDomain
from pypanther.rules.aws_vpc_flow.aws_vpc_healthy_log_status import AWSVPCHealthyLogStatus as AWSVPCHealthyLogStatus
from pypanther.rules.aws_vpc_flow.aws_vpc_inbound_traffic_port_allowlist import (
    AWSVPCInboundPortWhitelist as AWSVPCInboundPortWhitelist,
)
from pypanther.rules.aws_vpc_flow.aws_vpc_inbound_traffic_port_blocklist import (
    AWSVPCInboundPortBlacklist as AWSVPCInboundPortBlacklist,
)
from pypanther.rules.aws_vpc_flow.aws_vpc_ssh_allowed_signal import AWSVPCSSHAllowedSignal as AWSVPCSSHAllowedSignal
from pypanther.rules.aws_vpc_flow.aws_vpc_unapproved_outbound_dns import (
    AWSVPCUnapprovedOutboundDNS as AWSVPCUnapprovedOutboundDNS,
)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed


@panther_managed
class BoxEventTriggeredExternally(Rule):
    id = "Box.Event.Triggered.Externally-prototype"
    display_name = "Box event triggered by unknown or external user"
    enabled = False
    log_types = [LogType.BOX_EVENT]
    tags = ["Box", "Exfiltration:Exfiltration Over Web Service", "Configuration Required"]
    reports = {"MITRE ATT&CK": ["TA0010:T1567"]}
    default_severity = Severity.MEDIUM
    default_description = "An external user has triggered a box enterprise event.\n"
    default_reference = "https://support.box.com/hc/en-us/articles/8391393127955-Using-the-Enterprise-Event-Stream"
    default_runbook = "Investigate whether this user's activity is expected.\n"
    summary_attributes = ["ip_address"]
    threshold = 10
    # "@example.com"
    DOMAINS = {}

    def rule(self, event):
        # Check that all events are triggered by internal users
        if event.get("event_type") not in ("FAILED_LOGIN", "SHIELD_ALERT"):
            user = event.get("created_by", {})
            # user id 2 indicates an anonymous user
            if user.get("id", "") == "2":
                return True
            if self.DOMAINS:
                return bool(user.get("login") and (not any(user.get("login", "").endswith(x) for x in self.DOMAINS)))
        return False

    def title(self, event):
        return (
            f"External user [{event.deep_get('created_by', 'login', default='<UNKNOWN_USER>')}] triggered a box event."
        )

    tests = [
        RuleTest(
            name="Regular Event",
            expected_result=False,
            log={
                "type": "event",
                "additional_details": '{"key": "value"}',
                "created_by": {"id": "12345678", "type": "user", "login": "cat@example.com", "name": "Bob Cat"},
                "event_type": "DELETE",
            },
        ),
        RuleTest(
            name="Previewed Anonymously",
            expected_result=True,
            log={
                "created_by": {"id": "2", "type": "user", "name": "Unknown User"},
                "event_type": "PREVIEW",
                "type": "event",
                "ip_address": "1.2.3.4",
            },
        ),
        RuleTest(
            name="Missing Created By",
            expected_result=False,
            log={"event_type": "PREVIEW", "type": "event", "ip_address": "1.2.3.4"},
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed


@panther_managed
class DropboxUserDisabled2FA(Rule):
    default_description = "Dropbox user has disabled 2fa login"
    display_name = "Dropbox User Disabled 2FA"
    default_reference = "https://help.dropbox.com/account-access/enable-two-step-verification"
    default_severity = Severity.LOW
    log_types = [LogType.DROPBOX_TEAM_EVENT]
    id = "Dropbox.User.Disabled.2FA-prototype"

    def rule(self, event):
        return all(
            [
                event.deep_get("details", ".tag", default="") == "tfa_change_status_details",
                event.deep_get("details", "new_value", ".tag") == "disabled",
            ],
        )

    def title(self, event):
        actor = event.deep_get("actor", "user", "email", default="<EMAIL_NOT_FOUND>")
        target = event.deep_get("context", "email", default="<TARGET_NOT_FOUND>")
        return f"Dropbox: [{actor}] disabled 2FA for [{target}]."

    tests = [
        RuleTest(
            name="2FA Disabled",
            expected_result=True,
            log={
                "actor": {
                    "_tag": "user",
                    "user": {
                        "_tag": "team_member",
                        "account_id": "dbid:AAAAAAAAAAAAAAAA",
                        "display_name": "Alice Bob",
                        "email": "alice.bob@company.io",
                        "team_member_id": "dbmid:AABBBBBBBBBBBBBBBBBBBBBBB",
                    },
                },
                "context": {
                    "_tag": "team_member",
                    "account_id": "dbid:AAAAAAAAAAAAAAAA",
                    "display_name": "Alice Bob",
                    "email": "alice.bob@company.io",
                    "team_member_id": "dbmid:AABBBBBBBBBBBBBBBBBBBBBBB",
                },
                "details": {
                    ".tag": "tfa_change_status_details",
                    "new_value": {".tag": "disabled"},
                    "previous_value": {".tag": "authenticator"},
                    "used_rescue_code": True,
                },
                "event_category": {"_tag": "tfa"},
                "event_type": {
                    "_tag": "tfa_change_status",
                    "description": "Enabled/disabled/changed two-step verification setting",
                },
                "involve_non_team_member": False,
                "origin": {
                    "access_method": {
                        ".tag": "end_user",
                        "end_user": {".tag": "web", "session_id": "dbwsid:237034608707419186011941491025532848312"},
                    },
                    "geo_location": {"city": "Austin", "country": "US", "ip_address": "1.2.3.4", "region": "Texas"},
                },
                "p_any_emails": ["alice.bob@company.io"],
                "p_any_ip_addresses": ["1.2.3.4"],
                "p_any_usernames": ["Alice Bob"],
                "p_event_time": "2023-04-18 18:16:27",
                "p_log_type": "Dropbox.TeamEvent",
                "p_parse_time": "2023-04-18 18:18:46.808",
                "p_row_id": "0eb86fcfca9bb1cdce9defd217b8ac03",
                "p_schema_version": 0,
                "p_source_id": "b09c205e-42af-4933-8b18-b910985eb7fb",
                "p_source_label": "dropbox1",
                "timestamp": "2023-04-18 18:16:27",
            },
        ),
        RuleTest(
            name="Other",
            expected_result=False,
            log={
                "actor": {
                    "_tag": "user",
                    "user": {
                        "_tag": "team_member",
                        "account_id": "dbid:xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
                        "display_name": "user_name",
                        "email": "user@domain.com",
                        "team_member_id": "dbmid:xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
                    },
                },
                "context": {"_tag": "team"},
                "details": {
                    ".tag": "app_link_member_details",
                    "app_info": {
                        ".tag": "member_linked_app",
                        "app_id": "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
                        "display_name": "personal-dropbox-app-name",
                    },
                },
                "event_category": {"_tag": "apps"},
                "event_type": {"_tag": "app_link_member", "description": "Linked app for member"},
                "involve_non_team_member": False,
                "origin": {
                    "access_method": {".tag": "api", "request_id": "dbarod:xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"},
                    "geo_location": {
                        "city": "Los Angeles",
                        "country": "US",
                        "ip_address": "1.2.3.4",
                        "region": "California",
                    },
                },
                "timestamp": "2023-02-16 20:39:34",
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.aws import aws_rule_context


@panther_managed
class AWSSuspiciousSAMLActivity(Rule):
    default_description = (
        "Identifies when SAML activity has occurred in AWS. An adversary could gain backdoor access via SAML."
    )
    display_name = "AWS SAML Activity"
    default_reference = (
        "https://docs.aws.amazon.com/cognito/latest/developerguide/cognito-user-pools-managing-saml-idp-console.html"
    )
    default_severity = Severity.MEDIUM
    log_types = [LogType.AWS_CLOUDTRAIL]
    id = "AWS.Suspicious.SAML.Activity-prototype"
    SAML_ACTIONS = ["UpdateSAMLProvider", "CreateSAMLProvider", "DeleteSAMLProvider"]

    def rule(self, event):
        # Allow AWSSSO to manage
        if event.deep_get("userIdentity", "arn", default="").endswith(":assumed-role/AWSServiceRoleForSSO/AWS-SSO"):
            return False
        # Don't alert on errors such as EntityAlreadyExistsException and NoSuchEntity
        if event.get("errorCode"):
            return False
        return event.get("eventSource") == "iam.amazonaws.com" and event.get("eventName") in self.SAML_ACTIONS

    def title(self, event):
        return f"[{event.deep_get('userIdentity', 'arn')}] performed [{event.get('eventName')}] in account [{event.get('recipientAccountId')}]"

    def alert_context(self, event):
        return aws_rule_context(event)

    tests = [
        RuleTest(
            name="CreateSAMLProvider",
            expected_result=True,
            log={
                "awsRegion": "us-east-1",
                "eventID": "EID12345",
                "eventName": "CreateSAMLProvider",
                "eventSource": "iam.amazonaws.com",
                "eventTime": "2021-10-14 21:25:20",
                "eventType": "AwsApiCall",
                "eventVersion": "1.08",
                "managementEvent": True,
                "recipientAccountId": "0123456789",
                "requestID": "ABC1234",
                "sourceIPAddress": "1.2.3.4",
                "userAgent": "cloudformation.amazonaws.com",
                "userIdentity": {
                    "accessKeyId": "ABCDEFGHIJK",
                    "accountId": "0123456789",
                    "arn": "arn:aws:sts::0123456789:assumed-role/role/account",
                    "invokedBy": "cloudformation.amazonaws.com",
                    "principalId": "0123456789:AWSCloudFormation",
                    "sessionContext": {
                        "attributes": {"creationDate": "2021-10-14T21:25:20Z", "mfaAuthenticated": "false"},
                        "sessionIssuer": {
                            "accountId": "0123456789",
                            "arn": "arn:aws:iam::0123456789:role/ServiceRole",
                            "principalId": "ABCDEFGI0123",
                            "type": "Role",
                            "userName": "ServiceRole",
                        },
                        "webIdFederationData": {},
                    },
                    "type": "AssumedRole",
                },
            },
        ),
        RuleTest(
            name="DeleteSAMLProvider",
            expected_result=True,
            log={
                "awsRegion": "us-east-1",
                "eventID": "EID12345",
                "eventName": "DeleteSAMLProvider",
                "eventSource": "iam.amazonaws.com",
                "eventTime": "2021-10-14 21:25:20",
                "eventType": "AwsApiCall",
                "eventVersion": "1.08",
                "managementEvent": True,
                "recipientAccountId": "0123456789",
                "requestID": "ABC1234",
                "sourceIPAddress": "1.2.3.4",
                "userAgent": "cloudformation.amazonaws.com",
                "userIdentity": {
                    "accessKeyId": "ABCDEFGHIJK",
                    "accountId": "0123456789",
                    "arn": "arn:aws:sts::0123456789:assumed-role/role/account",
                    "invokedBy": "cloudformation.amazonaws.com",
                    "principalId": "0123456789:AWSCloudFormation",
                    "sessionContext": {
                        "attributes": {"creationDate": "2021-10-14T21:25:20Z", "mfaAuthenticated": "false"},
                        "sessionIssuer": {
                            "accountId": "0123456789",
                            "arn": "arn:aws:iam::0123456789:role/ServiceRole",
                            "principalId": "ABCDEFGI0123",
                            "type": "Role",
                            "userName": "ServiceRole",
                        },
                        "webIdFederationData": {},
                    },
                    "type": "AssumedRole",
                },
            },
        ),
        RuleTest(
            name="Non Target Event",
            expected_result=False,
            log={
                "awsRegion": "us-east-1",
                "eventID": "EID12345",
                "eventName": "ListAccessKeys",
                "eventSource": "iam.amazonaws.com",
                "eventTime": "2021-10-13 18:35:08",
                "eventType": "AwsApiCall",
                "eventVersion": "1.08",
                "managementEvent": True,
                "readOnly": True,
                "recipientAccountId": "0123456789",
                "requestID": "requestID12345",
                "sourceIPAddress": "1.2.3.4",
                "userAgent": "console.amazonaws.com",
                "userIdentity": {
                    "accessKeyId": "ABCDEFGHIJKLMNOP",
                    "accountId": "0123456789",
                    "arn": "arn:aws:iam::0123456789:user/bob",
                    "principalId": "ABCDEF012345",
                    "sessionContext": {
                        "attributes": {"creationDate": "2021-10-13T18:35:02Z", "mfaAuthenticated": "true"},
                        "sessionIssuer": {},
                        "webIdFederationData": {},
                    },
                    "type": "IAMUser",
                    "userName": "bob",
                },
            },
        ),
        RuleTest(
            name="UpdateSAMLProvider",
            expected_result=True,
            log={
                "awsRegion": "us-east-1",
                "eventID": "EID12345",
                "eventName": "UpdateSAMLProvider",
                "eventSource": "iam.amazonaws.com",
                "eventTime": "2021-10-14 21:25:20",
                "eventType": "AwsApiCall",
                "eventVersion": "1.08",
                "managementEvent": True,
                "recipientAccountId": "0123456789",
                "requestID": "ABC1234",
                "sourceIPAddress": "1.2.3.4",
                "userAgent": "cloudformation.amazonaws.com",
                "userIdentity": {
                    "accessKeyId": "ABCDEFGHIJK",
                    "accountId": "0123456789",
                    "arn": "arn:aws:sts::0123456789:assumed-role/role/account",
                    "invokedBy": "cloudformation.amazonaws.com",
                    "principalId": "0123456789:AWSCloudFormation",
                    "sessionContext": {
                        "attributes": {"creationDate": "2021-10-14T21:25:20Z", "mfaAuthenticated": "false"},
                        "sessionIssuer": {
                            "accountId": "0123456789",
                            "arn": "arn:aws:iam::0123456789:role/ServiceRole",
                            "principalId": "ABCDEFGI0123",
                            "type": "Role",
                            "userName": "ServiceRole",
                        },
                        "webIdFederationData": {},
                    },
                    "type": "AssumedRole",
                },
            },
        ),
        RuleTest(
            name="Activity from AWSSSO Service Managed Role",
            expected_result=False,
            log={
                "awsRegion": "us-east-1",
                "eventCategory": "Management",
                "eventName": "CreateSAMLProvider",
                "eventSource": "iam.amazonaws.com",
                "eventTime": "2022-12-12 21:46:17.000000000",
                "eventType": "AwsApiCall",
                "eventVersion": "1.08",
                "managementEvent": True,
                "p_alert_context": {
                    "awsRegion": "us-east-1",
                    "eventName": "CreateSAMLProvider",
                    "eventSource": "iam.amazonaws.com",
                    "recipientAccountId": "123412341234",
                    "sourceIPAddress": "sso.amazonaws.com",
                    "userAgent": "sso.amazonaws.com",
                    "userIdentity": {
                        "accessKeyId": "ASIAXXXXNLMHSP3MFXX",
                        "accountId": "123412341234",
                        "arn": "arn:aws:sts::123412341234:assumed-role/AWSServiceRoleForSSO/AWS-SSO",
                        "invokedBy": "sso.amazonaws.com",
                        "principalId": "AROAT7BCMNLMONMOFFFFF:AWS-SSO",
                        "sessionContext": {
                            "attributes": {"creationDate": "2022-12-12T21:46:16Z", "mfaAuthenticated": "false"},
                            "sessionIssuer": {
                                "accountId": "123412341234",
                                "arn": "arn:aws:iam::123412341234:role/aws-service-role/sso.amazonaws.com/AWSServiceRoleForSSO",
                                "principalId": "AROAT7BCMNLMONMOFFFFF",
                                "type": "Role",
                                "userName": "AWSServiceRoleForSSO",
                            },
                            "webIdFederationData": {},
                        },
                        "type": "AssumedRole",
                    },
                },
                "p_alert_creation_time": "2022-12-12 21:51:37.115853000",
                "p_alert_update_time": "2022-12-12 21:51:37.115853000",
                "p_any_aws_account_ids": ["123412341234"],
                "p_any_aws_arns": [
                    "arn:aws:iam::123412341234:role/aws-service-role/sso.amazonaws.com/AWSServiceRoleForSSO",
                    "arn:aws:iam::123412341234:saml-provider/AWSSSO_abdf34fd171b4a7e_DO_NOT_DELETE",
                    "arn:aws:sts::123412341234:assumed-role/AWSServiceRoleForSSO/AWS-SSO",
                ],
                "p_any_domain_names": ["sso.amazonaws.com"],
                "p_any_trace_ids": ["ASIAXXXXNLMHSP3MFXX"],
                "p_any_usernames": ["AWSServiceRoleForSSO"],
                "p_event_time": "2022-12-12 21:46:17.000000000",
                "p_log_type": "AWS.CloudTrail",
                "p_parse_time": "2022-12-12 21:49:13.694384486",
                "p_rule_id": "AWS.Suspicious.SAML.Activity",
                "p_source_label": "YourOrg - Cloudtrail - Label",
                "readOnly": False,
                "recipientAccountId": "123412341234",
                "requestID": "cb89df1f-6019-427f-9a69-00b8b904ce0d",
                "requestParameters": {
                    "name": "AWSSSO_abdf34fd171b4a7e_DO_NOT_DELETE",
                    "sAMLMetadataDocument": '<?xml version="1.0" encoding="UTF-8"?></xml>',
                },
                "responseElements": {
                    "sAMLProviderArn": "arn:aws:iam::123412341234:saml-provider/AWSSSO_abdf34fd171b4a7e_DO_NOT_DELETE",
                },
                "sourceIPAddress": "sso.amazonaws.com",
                "userAgent": "sso.amazonaws.com",
                "userIdentity": {
                    "accessKeyId": "ASIAXXXXNLMHSP3MFXX",
                    "accountId": "123412341234",
                    "arn": "arn:aws:sts::123412341234:assumed-role/AWSServiceRoleForSSO/AWS-SSO",
                    "invokedBy": "sso.amazonaws.com",
                    "principalId": "AROAT7BCMNLMONMOFFFFF:AWS-SSO",
                    "sessionContext": {
                        "attributes": {"creationDate": "2022-12-12T21:46:16Z", "mfaAuthenticated": "false"},
                        "sessionIssuer": {
                            "accountId": "123412341234",
                            "arn": "arn:aws:iam::123412341234:role/aws-service-role/sso.amazonaws.com/AWSServiceRoleForSSO",
                            "principalId": "AROAT7BCMNLMONMOFFFFF",
                            "type": "Role",
                            "userName": "AWSServiceRoleForSSO",
                        },
                        "webIdFederationData": {},
                    },
                    "type": "AssumedRole",
                },
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed


@panther_managed
class DropboxOwnershipTransfer(Rule):
    default_description = "Dropbox ownership of a document or folder has been transferred."
    display_name = "Dropbox Document/Folder Ownership Transfer"
    default_reference = "https://help.dropbox.com/share/owner"
    default_severity = Severity.MEDIUM
    log_types = [LogType.DROPBOX_TEAM_EVENT]
    id = "Dropbox.Ownership.Transfer-prototype"

    def rule(self, event):
        return "Transferred ownership " in event.deep_get("event_type", "description", default="")

    def title(self, event):
        actor = event.deep_get("actor", "user", "email", default="<EMAIL_NOT_FOUND>")
        previous_owner = event.deep_get("details", "previous_owner_email", default="<PREVIOUS_OWNER_NOT_FOUND>")
        new_owner = event.deep_get("details", "new_owner_email", default="<NEW_OWNER_NOT_FOUND>")
        assets = event.get("assets", [{}])
        asset = [a.get("display_name", "<ASSET_NOT_FOUND>") for a in assets]
        return f"Dropbox: [{actor}] transferred ownership of [{asset}]from [{previous_owner}] to [{new_owner}]."

    def severity(self, event):
        new_owner_domain = event.deep_get("details", "new_owner_email", default="@").split("@")[-1]
        previous_owner_domain = event.deep_get("details", "previous_owner_email", default="@").split("@")[-1]
        if new_owner_domain != previous_owner_domain:
            return "DEFAULT"
        return "LOW"

    tests = [
        RuleTest(
            name="Folder Ownership Transfer to External",
            expected_result=True,
            log={
                "actor": {
                    "_tag": "user",
                    "user": {
                        "_tag": "team_member",
                        "account_id": "dbid:AAAAAAAAAAAAAAAAAAAAA",
                        "display_name": "Alice Bob",
                        "email": "alice.bob@company.io",
                        "team_member_id": "dbmid:BBBBBBBBBBBBBBBBBBBBBB",
                    },
                },
                "assets": [
                    {
                        ".tag": "folder",
                        "display_name": "test1",
                        "path": {
                            "contextual": "/Alice Bob/test1",
                            "namespace_relative": {"is_shared_namespace": True, "ns_id": "12345"},
                        },
                    },
                ],
                "context": {
                    "_tag": "team_member",
                    "account_id": "dbid:AAAAAAAAAAAAAAAAAAAAA",
                    "display_name": "Alice Bob",
                    "email": "alice.bob@company.io",
                    "team_member_id": "dbmid:BBBBBBBBBBBBBBBBBBBBBB",
                },
                "details": {
                    ".tag": "shared_folder_transfer_ownership_details",
                    "new_owner_email": "david.davidson@company.io",
                    "previous_owner_email": "alice.bob@company.io",
                },
                "event_category": {"_tag": "sharing"},
                "event_type": {
                    "_tag": "shared_folder_transfer_ownership",
                    "description": "Transferred ownership of shared folder to another member",
                },
                "involve_non_team_member": False,
                "origin": {
                    "access_method": {
                        ".tag": "end_user",
                        "end_user": {".tag": "web", "session_id": "dbwsid:237034608707419186011941491025532848312"},
                    },
                    "geo_location": {"city": "Austin", "country": "US", "ip_address": "1.2.3.4", "region": "Texas"},
                },
                "p_any_emails": ["alice.bob@company.io", "david.davidson@company.io"],
                "p_any_ip_addresses": ["1.2.3.4"],
                "p_any_usernames": ["Alice Bob", "company"],
                "p_event_time": "2023-04-18 18:54:15",
                "p_log_type": "Dropbox.TeamEvent",
                "p_parse_time": "2023-04-18 18:56:47.418",
                "p_row_id": "0eb86fcfca9bb1cdce9defd217e1cd04",
                "p_schema_version": 0,
                "p_source_id": "b09c205e-42af-4933-8b18-b910985eb7fb",
                "p_source_label": "dropbox1",
                "participants": [
                    {
                        "user": {
                            "_tag": "team_member",
                            "account_id": "dbid:ABCD",
                            "display_name": "company",
                            "email": "david.davidson@company.io",
                            "team_member_id": "dbmid:DEFG",
                        },
                    },
                ],
                "timestamp": "2023-04-18 18:54:15",
            },
        ),
        RuleTest(
            name="Other",
            expected_result=False,
            log={
                "actor": {
                    "_tag": "user",
                    "user": {
                        "_tag": "team_member",
                        "account_id": "dbid:AAAAAAAAAAAAAAAA",
                        "display_name": "Alice Bob",
                        "email": "alice.bob@company.io",
                        "team_member_id": "dbmid:AABBBBBBBBBBBBBBBBBBBBBBB",
                    },
                },
                "context": {
                    "_tag": "team_member",
                    "account_id": "dbid:AAAAAAAAAAAAAAAA",
                    "display_name": "Alice Bob",
                    "email": "alice.bob@company.io",
                    "team_member_id": "dbmid:AABBBBBBBBBBBBBBBBBBBBBBB",
                },
                "details": {
                    ".tag": "tfa_change_status_details",
                    "new_value": {".tag": "disabled"},
                    "previous_value": {".tag": "authenticator"},
                    "used_rescue_code": True,
                },
                "event_category": {"_tag": "tfa"},
                "event_type": {
                    "_tag": "tfa_change_status",
                    "description": "Enabled/disabled/changed two-step verification setting",
                },
                "involve_non_team_member": False,
                "origin": {
                    "access_method": {
                        ".tag": "end_user",
                        "end_user": {".tag": "web", "session_id": "dbwsid:237034608707419186011941491025532848312"},
                    },
                    "geo_location": {"city": "Austin", "country": "US", "ip_address": "1.2.3.4", "region": "Texas"},
                },
                "p_any_emails": ["alice.bob@company.io"],
                "p_any_ip_addresses": ["1.2.3.4"],
                "p_any_usernames": ["Alice Bob"],
                "p_event_time": "2023-04-18 18:16:27",
                "p_log_type": "Dropbox.TeamEvent",
                "p_parse_time": "2023-04-18 18:18:46.808",
                "p_row_id": "0eb86fcfca9bb1cdce9defd217b8ac03",
                "p_schema_version": 0,
                "p_source_id": "b09c205e-42af-4933-8b18-b910985eb7fb",
                "p_source_label": "dropbox1",
                "timestamp": "2023-04-18 18:16:27",
            },
        ),
        RuleTest(
            name="Folder Ownership Transfer to Internal",
            expected_result=True,
            log={
                "actor": {
                    "_tag": "user",
                    "user": {
                        "_tag": "team_member",
                        "account_id": "dbid:AAAAAAAAAAAAAAAAAAAAA",
                        "display_name": "Alice Bob",
                        "email": "alice.bob@company.io",
                        "team_member_id": "dbmid:BBBBBBBBBBBBBBBBBBBBBB",
                    },
                },
                "assets": [
                    {
                        ".tag": "folder",
                        "display_name": "test1",
                        "path": {
                            "contextual": "/Alice Bob/test1",
                            "namespace_relative": {"is_shared_namespace": True, "ns_id": "12345"},
                        },
                    },
                ],
                "context": {
                    "_tag": "team_member",
                    "account_id": "dbid:AAAAAAAAAAAAAAAAAAAAA",
                    "display_name": "Alice Bob",
                    "email": "alice.bob@company.io",
                    "team_member_id": "dbmid:BBBBBBBBBBBBBBBBBBBBBB",
                },
                "details": {
                    ".tag": "shared_folder_transfer_ownership_details",
                    "new_owner_email": "david.davidson@example.com",
                    "previous_owner_email": "alice.bob@company.io",
                },
                "event_category": {"_tag": "sharing"},
                "event_type": {
                    "_tag": "shared_folder_transfer_ownership",
                    "description": "Transferred ownership of shared folder to another member",
                },
                "involve_non_team_member": False,
                "origin": {
                    "access_method": {
                        ".tag": "end_user",
                        "end_user": {".tag": "web", "session_id": "dbwsid:237034608707419186011941491025532848312"},
                    },
                    "geo_location": {"city": "Austin", "country": "US", "ip_address": "1.2.3.4", "region": "Texas"},
                },
                "p_any_emails": ["alice.bob@company.io", "david.davidson@example.com"],
                "p_any_ip_addresses": ["1.2.3.4"],
                "p_any_usernames": ["Alice Bob", "company"],
                "p_event_time": "2023-04-18 18:54:15",
                "p_log_type": "Dropbox.TeamEvent",
                "p_parse_time": "2023-04-18 18:56:47.418",
                "p_row_id": "0eb86fcfca9bb1cdce9defd217e1cd04",
                "p_schema_version": 0,
                "p_source_id": "b09c205e-42af-4933-8b18-b910985eb7fb",
                "p_source_label": "dropbox1",
                "participants": [
                    {
                        "user": {
                            "_tag": "team_member",
                            "account_id": "dbid:ABCD",
                            "display_name": "company",
                            "email": "david.davidson@example.com",
                            "team_member_id": "dbmid:DEFG",
                        },
                    },
                ],
                "timestamp": "2023-04-18 18:54:15",
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.aws import aws_cloudtrail_success, aws_rule_context


@panther_managed
class AWSIAMCredentialsUpdated(Rule):
    id = "AWS.IAM.CredentialsUpdated-prototype"
    display_name = "New IAM Credentials Updated"
    log_types = [LogType.AWS_CLOUDTRAIL]
    reports = {"MITRE ATT&CK": ["TA0003:T1098"]}
    tags = ["AWS", "Identity & Access Management", "Persistence:Account Manipulation"]
    default_severity = Severity.INFO
    default_description = "A console password, access key, or user has been created."
    default_runbook = "This rule is purely informational, there is no action needed."
    default_reference = "https://docs.aws.amazon.com/IAM/latest/UserGuide/list_identityandaccessmanagement.html"
    summary_attributes = ["eventName", "userAgent", "sourceIpAddress", "recipientAccountId", "p_any_aws_arns"]
    UPDATE_EVENTS = {"ChangePassword", "CreateAccessKey", "CreateLoginProfile", "CreateUser"}

    def rule(self, event):
        return event.get("eventName") in self.UPDATE_EVENTS and aws_cloudtrail_success(event)

    def dedup(self, event):
        return event.deep_get("userIdentity", "userName", default="<UNKNOWN_USER>")

    def title(self, event):
        return f"{event.deep_get('userIdentity', 'type')} [{event.deep_get('userIdentity', 'arn')}] has updated their IAM credentials"

    def alert_context(self, event):
        return aws_rule_context(event)

    tests = [
        RuleTest(
            name="User Password Was Changed",
            expected_result=True,
            log={
                "eventVersion": "1.05",
                "userIdentity": {
                    "type": "IAMUser",
                    "principalId": "AAAAIIIIIIU74NPJW5K76",
                    "arn": "arn:aws:iam::123456789012:user/test_user",
                    "accountId": "123456789012",
                    "accessKeyId": "AAAAIIIIIIU74NPJW5K76",
                    "userName": "test_user",
                    "sessionContext": {
                        "attributes": {"mfaAuthenticated": "true", "creationDate": "2019-12-31T01:50:17Z"},
                    },
                    "invokedBy": "signin.amazonaws.com",
                },
                "eventTime": "2019-12-31T01:50:46Z",
                "eventSource": "iam.amazonaws.com",
                "eventName": "ChangePassword",
                "awsRegion": "us-east-1",
                "sourceIPAddress": "64.25.27.224",
                "userAgent": "signin.amazonaws.com",
                "requestParameters": None,
                "responseElements": None,
                "requestID": "a431f05e-67e1-11ea-bc55-0242ac130003",
                "eventID": "a431f05e-67e1-11ea-bc55-0242ac130003",
                "readOnly": False,
                "eventType": "AwsApiCall",
                "recipientAccountId": "123456789012",
            },
        ),
        RuleTest(
            name="MFA Device Was Created",
            expected_result=False,
            log={
                "eventVersion": "1.05",
                "userIdentity": {
                    "type": "IAMUser",
                    "principalId": "AAAAIIIIIIU74NPJW5K76",
                    "arn": "arn:aws:iam::123456789012:user/test_user",
                    "accountId": "123456789012",
                    "accessKeyId": "AAAAIIIIIIU74NPJW5K76",
                    "userName": "test_user",
                    "sessionContext": {
                        "attributes": {"mfaAuthenticated": "true", "creationDate": "2019-12-31T01:50:17Z"},
                    },
                    "invokedBy": "signin.amazonaws.com",
                },
                "eventTime": "2019-12-31T01:50:46Z",
                "eventSource": "iam.amazonaws.com",
                "eventName": "CreateVirtualMFADevice",
                "awsRegion": "us-east-1",
                "sourceIPAddress": "64.25.27.224",
                "userAgent": "signin.amazonaws.com",
                "requestParameters": None,
                "responseElements": None,
                "requestID": "a431f05e-67e1-11ea-bc55-0242ac130003",
                "eventID": "a431f05e-67e1-11ea-bc55-0242ac130003",
                "readOnly": False,
                "eventType": "AwsApiCall",
                "recipientAccountId": "123456789012",
            },
        ),
        RuleTest(
            name="User Password Change Error",
            expected_result=False,
            log={
                "eventVersion": "1.05",
                "errorCode": "PasswordPolicyViolation",
                "userIdentity": {
                    "type": "IAMUser",
                    "principalId": "AAAAIIIIIIU74NPJW5K76",
                    "arn": "arn:aws:iam::123456789012:user/test_user",
                    "accountId": "123456789012",
                    "accessKeyId": "AAAAIIIIIIU74NPJW5K76",
                    "userName": "test_user",
                    "sessionContext": {
                        "attributes": {"mfaAuthenticated": "true", "creationDate": "2019-12-31T01:50:17Z"},
                    },
                    "invokedBy": "signin.amazonaws.com",
                },
                "eventTime": "2019-12-31T01:50:46Z",
                "eventSource": "iam.amazonaws.com",
                "eventName": "ChangePassword",
                "awsRegion": "us-east-1",
                "sourceIPAddress": "64.25.27.224",
                "userAgent": "signin.amazonaws.com",
                "requestParameters": None,
                "responseElements": None,
                "requestID": "a431f05e-67e1-11ea-bc55-0242ac130003",
                "eventID": "a431f05e-67e1-11ea-bc55-0242ac130003",
                "readOnly": False,
                "eventType": "AwsApiCall",
                "recipientAccountId": "123456789012",
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.aws import aws_rule_context


@panther_managed
class AWSIAMAccessKeyCompromised(Rule):
    id = "AWS.IAM.AccessKeyCompromised-prototype"
    display_name = "AWS IAM Access Key Compromise Detection"
    log_types = [LogType.AWS_CLOUDTRAIL]
    reports = {"MITRE ATT&CK": ["TA0006:T1552"]}
    tags = ["AWS", "Credential Access:Unsecured Credentials"]
    default_severity = Severity.HIGH
    default_description = "This alert occurs when AWS has detected exposed credentials. It attaches a policy to deny certain actions, effectively quarantining those credentials, and is accompanied by a support case with instructions for detaching the policy."
    default_runbook = "Determine the IAM user who owns the key, rotate/disable/delete the key, and then investigate which actions were taken by the compromised key to determine scope and if any remediation is needed."
    default_reference = "https://docs.github.com/en/code-security/secret-scanning/about-secret-scanning"
    EXPOSED_CRED_POLICIES = {
        "AWSExposedCredentialPolicy_DO_NOT_REMOVE",
        "AWSCompromisedKeyQuarantine",
        "AWSCompromisedKeyQuarantineV2",
        "AWSCompromisedKeyQuarantineV3",
    }

    def rule(self, event):
        if event.get("eventName") != "PutUserPolicy":
            return False
        request_params = event.get("requestParameters") or {}
        if request_params.get("policyName") not in self.EXPOSED_CRED_POLICIES:
            return False
        return True

    def title(self, event):
        user_name = event.deep_get("userIdentity", "userName")
        access_key_id = event.deep_get("userIdentity", "accessKeyId")
        return f"[{user_name}]'s AWS IAM Access Key ID [{access_key_id}] was exposed and quarantined by AWS"

    def alert_context(self, event):
        return aws_rule_context(event)

    tests = [
        RuleTest(
            name="An AWS IAM Access Key was Compromised",
            expected_result=True,
            log={
                "eventSource": "iam.amazonaws.com",
                "recipientAccountId": "123456789012",
                "responseElements": None,
                "userIdentity": {
                    "type": "IAMUser",
                    "userName": "compromised_user",
                    "principalId": "XXXXXXXXXXXXXXXXXXX",
                    "accessKeyId": "XXXXXXXXXXXXXXXXXXXXX",
                    "arn": "arn:aws:iam::123456789012:user/compromised_user",
                    "accountId": "123456789012",
                },
                "eventName": "PutUserPolicy",
                "eventVersion": "1.05",
                "userAgent": "aws-internal/3 aws-sdk-java/1.11.706 Linux/4.9.184-0.1.ac.235.83.329.metal1.x86_64 OpenJDK_64-Bit_Server_VM/25.242-b08 java/1.8.0_242 vendor/Oracle_Corporation",
                "requestParameters": {
                    "policyDocument": '{"Version":"2012-10-17","Statement":[{"Sid":"Stmt1538161409","Effect":"Deny","Action":["lambda:CreateFunction","iam:AttachUserPolicy","iam:PutUserPolicy","organizations:InviteAccountToOrganization","ec2:RunInstances","iam:DetachUserPolicy","iam:CreateUser","lightsail:Create*","lightsail:Update*","ec2:StartInstances","ec2:RequestSpotInstances","iam:ChangePassword","iam:CreateLoginProfile","organizations:CreateOrganization","organizations:CreateAccount","lightsail:Delete*","iam:AttachGroupPolicy","iam:CreateAccessKey","iam:UpdateUser","iam:UpdateAccountPasswordPolicy","iam:DeleteUserPolicy","iam:PutUserPermissionsBoundary","iam:UpdateAccessKey","lightsail:DownloadDefaultKeyPair","iam:CreateInstanceProfile","lightsail:Start*","lightsail:GetInstanceAccessDetails","iam:CreateRole","iam:PutGroupPolicy","iam:AttachRolePolicy"],"Resource":["*"]}]}',
                    "userName": "compromised_user",
                    "policyName": "AWSCompromisedKeyQuarantineV3",
                },
                "eventID": "1c2a53d1-58cc-41b3-85b8-bd7565370e0d",
                "eventType": "AwsApiCall",
                "sourceIPAddress": "72.21.217.97",
                "awsRegion": "us-east-1",
                "requestID": "27ca92a5-61cc-44aa-b875-042a25310064",
                "eventTime": "2020-04-10T06:22:08Z",
            },
        ),
        RuleTest(
            name="Request Param is null",
            expected_result=False,
            log={
                "eventSource": "iam.amazonaws.com",
                "recipientAccountId": "123456789012",
                "responseElements": None,
                "userIdentity": {
                    "type": "IAMUser",
                    "userName": "compromised_user",
                    "principalId": "XXXXXXXXXXXXXXXXXXX",
                    "accessKeyId": "XXXXXXXXXXXXXXXXXXXXX",
                    "arn": "arn:aws:iam::123456789012:user/compromised_user",
                    "accountId": "123456789012",
                },
                "eventName": "PutUserPolicy",
                "eventVersion": "1.05",
                "userAgent": "aws-internal/3 aws-sdk-java/1.11.706 Linux/4.9.184-0.1.ac.235.83.329.metal1.x86_64 OpenJDK_64-Bit_Server_VM/25.242-b08 java/1.8.0_242 vendor/Oracle_Corporation",
                "requestParameters": None,
                "eventID": "1c2a53d1-58cc-41b3-85b8-bd7565370e0d",
                "eventType": "AwsApiCall",
                "sourceIPAddress": "72.21.217.97",
                "awsRegion": "us-east-1",
                "requestID": "27ca92a5-61cc-44aa-b875-042a25310064",
                "eventTime": "2020-04-10T06:22:08Z",
            },
        ),
    ]
